# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-02-28 18:13:42
# @Email:
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-21 14:15:08
# @Descriptions

import os.path
import numpy as np
from typing import List

from pydub import AudioSegment
from scipy.io.wavfile import write, read
from moviepy.video.io.VideoFileClip import VideoFileClip
from pydub.exceptions import CouldntDecodeError
from gailbot.shared.exception.serviceException import IncompatibleFileFormat

from gailbot.shared.utils.general import delete, get_name, copy, get_extension, run_cmd
from gailbot.shared.utils.logger import makelogger
from gailbot.shared.utils.media import AudioHandler

logger = makelogger("converter")


def _raise_bad_format(path: str, err: Exception):
    logger.error(err, exc_info=True)
    extension = get_extension(path)
    raise IncompatibleFileFormat(path, extension)


def _convert_audio(path, tgt_parent_directory) -> List[str]:
    tgt = os.path.join(tgt_parent_directory, get_name(path) + ".wav")
    command = ["ffmpeg", "-i", f'"{path}"', "-acodec", "pcm_s16le", f'"{tgt}"']
    if os.path.isfile(tgt):
        delete(tgt)
    result = run_cmd(command)
    if result == 0:
        return [tgt]
    else:
        raise IncompatibleFileFormat(path, get_extension(path))


def _convert_video(path, tgt_parent_directory) -> List[str]:
    try:
        video_clip = VideoFileClip(path)
        audio_clip = video_clip.audio
        name = get_name(path)
        output = os.path.join(tgt_parent_directory, f"{name}.wav")
        audio_clip.write_audiofile(output, codec="pcm_s16le", fps=44100)
        video_clip.close()
        audio_clip.close()
        return [output]
    # VideoFileClip raises `KeyError: 'video_fps'` when given an audio file
    # (wav) instead of video (mov). We catch that and raise our own exception.
    except KeyError as e:
        _raise_bad_format(path, e)


def _convert_mxf(path, tgt_parent_directory) -> List[str]:
    name = get_name(path)
    video_clip = VideoFileClip(path)
    audio_clip = video_clip.audio

    # number of bytes to encode the sound:
    # 1 for 8bit sound, 2 for 16bit, 4 for 32bit
    sound_data = audio_clip.to_soundarray(fps=44100, quantize=True, nbytes=2)

    # extract each audio channel into a separate wav file
    num_channels = audio_clip.nchannels
    outputs = []
    for i in range(num_channels):
        output_path = os.path.join(tgt_parent_directory, f"{name}{i + 1}.wav")
        channel_audio = sound_data[:, i]
        # check that the current channel isn't a duplicate of another channel
        skip_channel = any(
            np.array_equal(channel_audio, sound_data[:, j]) for j in range(i)
        )
        if not skip_channel:
            write(output_path, audio_clip.fps, channel_audio)
            outputs.append(output_path)

    audio_clip.close()
    video_clip.close()
    return outputs

def _convert_wav(path, tgt_parent_directory):
    try:
        # if this succeeds, it really was a WAV — just copy it
        read(path)
        copy_path = os.path.join(tgt_parent_directory,
                                 os.path.basename(path))
        copy(src_path=path, tgt_path=copy_path)
        return [copy_path]
    except ValueError:
        # not a pure RIFF WAV, let's *attempt* a generic transcode instead
        try:
            return _convert_audio(path=path,
                                  tgt_parent_directory=tgt_parent_directory)
        except IncompatibleFileFormat as conv_err:
            # now we truly give up
            _raise_bad_format(path, conv_err)

class ConvertFun:
    convert_funs = {
        "mp3": _convert_audio,
        "mp4": _convert_video,
        "aac": _convert_audio,
        "m4a": _convert_audio,
        "wav": _convert_wav,
        "mov": _convert_video,
        "avi": _convert_video,
        "3gp": _convert_video,
        "mxf": _convert_mxf,
    }

    @staticmethod
    def convert_to_wav(path, tgt_parent_directory) -> List[str]:
        """
        convert the path to wav file, converted file should bed stored in tgt_parent_directory,
        and return the path of the converted file

        Parameters
        ----------
        path
        tgt_parent_directory

        Returns
        -------

        """
        file_format = get_extension(path).lower()
        return ConvertFun.convert_funs[file_format](path, tgt_parent_directory)

    @staticmethod
    def compatible_video_formats():
        return ["mp4", "mov", "3gp", "mxf", "avi"]

    @staticmethod
    def compatible_audio_formats():
        return ["mp3", "aac", "wav", "m4a"]

    @staticmethod
    def compatible_formats():
        return list(ConvertFun.convert_funs.keys())



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-07-17 19:21:51
# @Last Modified by:   Hannah Shader
# @Last Modified time: 2023-08-16 14:35:22

from .service import load_service_config, load_default_setting



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-02-24 10:12:37
# @Email:
# @Last Modified by:   Vivian Li 
# @Last Modified time: 2024-03-03 15:13:58
# @Descriptions: convert a transcribed directory source to a singleton list of
#   one payload


import os.path
from datetime import datetime
from typing import List

from .converter import Converter, ConverterType
from ..payloadObject import PayloadObject
from ..payloadObject.transcribedDirPayload import TranscribedDirPayload
from ...profileManager import ProfileObject
from ...shared.utils.general import get_name, copy
from ...shared.utils.logger import makelogger
from ...sourceManager.watcher import Watcher
from ...workspace.directory_structure import OutputFolder, TemporaryFolder

logger = makelogger("converter")


class TranscribedDirConverter(Converter):
    accepted_forms = ["/"]
    converter_type = ConverterType.TranscribedResult

    @staticmethod
    def is_accepted_form(path: str):
        return os.path.isdir(path) and os.path.isfile(
            os.path.join(path, OutputFolder.GB_RESULT_SIGNATURE)
        )

    @staticmethod
    def convert(
            path: str, output_dir: str, profile: ProfileObject, watchers: List[Watcher]
    ) -> List[PayloadObject]:
        name = get_name(path)
        time = datetime.now().strftime("%H-%M-%S")
        temp_path = TemporaryFolder(f"{name}_{time}")
        copy_path = os.path.join(temp_path.data_copy, name)
        copy(path, copy_path)

        return [TranscribedDirPayload(original_source=path,
                                      data_files=[copy_path],
                                      temp_dir_struct=temp_path,
                                      output_path=output_dir,
                                      profile=profile,
                                      watchers=watchers)]



# -*- coding: utf-8 -*-
# @Author: Lakshita Jain
# @Date:   2023-10-03 15:01:18
# @Last Modified by:   Joanne Fan
# @Last Modified time: 2024-01-25 23:18:08
import pytest
from gailbot import GailBot
from gailbot.services.organizer.settings.objects.settingObject import SettingObject
from typing import List, Dict, Union, Tuple, Callable
from gailbot.plugins.suite import PluginSuite


def test_clear_workspace(gb: GailBot, APP_ROOT, TEST_ROOT, OUTPUT_ROOT):
    result = gb.clear_workspace()
    print(APP_ROOT, TEST_ROOT, OUTPUT_ROOT)
    assert result == True


def test_reset_workspace(gb: GailBot):
    result = gb.reset_workspace()
    assert result == True


###########################################################################
# Sources                                                                 #
###########################################################################


@pytest.mark.parametrize(
    "source, setting, overwrite, expected",
    [
        ("hello1", "SETTING_OBJECT_WHISPER", True, True),
        ("hello2", "SETTING_OBJECT_WHISPER", False, True),
        ("fake_source", "SETTING_OBJECT_WHISPER", True, False),
        ("fake_source", "SETTING_OBJECT_WHISPERX", False, False),
    ],
)
def test_apply_profile_to_source(
    gb_with_2_sources: GailBot,
    source: str,
    setting: SettingObject,
    overwrite: bool,
    expected: bool,
    request,
):
    """
    Purpose: test applying a setting profile to source
    Expected Output: True if applied, false if not
    """
    result = gb_with_2_sources.apply_profile_to_source(
        source, request.getfixturevalue(setting), overwrite
    )
    assert result == expected


@pytest.mark.parametrize(
    "source_path, output_dir, expected",
    [
        ("HELLO_1", "OUTPUT_ROOT", "hello1"),
        ("FAKE_SOURCE", "OUTPUT_ROOT", "nonexistent_source"),
        ("HELLO_2", "OUTPUT_ROOT", "hello2"),
    ],
)
def test_add_source(
    gb: GailBot, source_path: str, output_dir: str, expected: bool, request
):
    """
    Purpose: test adding a source
    Expected Output: source name if added, false if not
    """
    result = gb.add_source(
        request.getfixturevalue(source_path), request.getfixturevalue(output_dir)
    )
    assert result == expected
    assert isinstance(result, str or bool)


@pytest.mark.parametrize(
    "src_output_pairs, expected",
    [
        (["SOURCE_LIST", True]),
    ],
)
def test_add_sources(
    gb: GailBot, src_output_pairs: List[Tuple[str, str]], expected: bool, request
):
    """
    Purpose: test adding multiple sources
    Expected Output: True if all successfully added, false if not
    """
    result = gb.add_sources(request.getfixturevalue(src_output_pairs))
    assert result == expected


@pytest.mark.parametrize(
    "name, expected",
    [
        ("hello1", True),
        ("hello2", True),
        ("nonexistent_source", False),
    ],
)
def test_is_source(gb_with_2_sources: GailBot, name: str, expected: bool):
    """
    Purpose: test if a given source name is a source
    Expected Output: True if it is a soy=urce, false if not
    """
    result = gb_with_2_sources.is_source(name)
    assert result == expected


@pytest.mark.parametrize(
    "name",
    [
        ("hello1"),
    ],
)
def test_get_source_output_directory(gb_with_2_sources: GailBot, name: str):
    """
    Purpose: test getting output directory corresponding to a given source name
    Expected Output: string of output directory path
    """
    result = gb_with_2_sources.get_source_output_directory(name)
    print(result)


@pytest.mark.parametrize(
    "name, expected",
    [("hello1", True), ("hello2", True), ("nonexistent_source", False)],
)
def test_remove_source(gb_with_2_sources: GailBot, name: str, expected: str):
    """
    Purpose: test removing a source
    Expected Output: true if removed, false if not
    """
    result = gb_with_2_sources.remove_source(name)
    assert result == expected


@pytest.mark.parametrize(
    "source_names, expected",
    [
        (["hello1", "hello2"], True),
        (["source_3"], False),
        (["source_1", "source_4"], False),
    ],
)
def test_remove_sources(
    gb_with_2_sources: GailBot, source_names: List[str], expected: bool
):
    """
    Purpose: test removing multiple sources
    Expected Output: True if successfully removed, false if not
    """
    result = gb_with_2_sources.remove_sources(source_names)
    assert result == expected


@pytest.mark.parametrize(
    "source_name, expected",
    [
        ("hello1", "WHISPER_PROFILE"),
        ("hello2", "WHISPERX_PROFILE"),
        ("fake_source", False),
    ],
)
def test_get_source_profile_dict(
    gb_with_sources_and_settings: GailBot, source_name: str, expected, request
):
    """
    Purpose: test getting profile dict corresponding to a source
    Expected Output: profile dict
    """
    result = gb_with_sources_and_settings.get_source_profile_data(source_name)
    if result == False:
        assert result == expected
    else:
        assert result == request.getfixturevalue(expected)


def test_clear_source_memory(gb_with_2_sources: GailBot):
    """
    Purpose: test clearing all sources
    Expected Output: True
    """
    result = gb_with_2_sources.clear_source_memory()
    assert result == True


# TODO: function not in API

# def test_get_all_source_names(gb_with_2_sources: GailBot):
#     """
#     Purpose: test getting names of all sources
#     Expected Output: list of source names
#     """
#     result = gb_with_2_sources.get_all_source_names()
#     assert result == ["hello1", "hello2", "hello_video"]


@pytest.mark.parametrize(
    "source_name, expected",
    [
        ("hello1", "test"),
        ("hello2", "test2"),
        ("source_3", False),
        ("source_4", False),
    ],
)
def test_source_profile_name(
    gb_with_sources_and_settings: GailBot, source_name: str, expected
):
    """
    Purpose: test getting names of source profiles
    Expected Output: profile name
    """
    result = gb_with_sources_and_settings.source_profile_name(source_name)
    assert result == expected


###########################################################################
# Transcribe                                                              #
###########################################################################


@pytest.mark.parametrize(
    "source_name",
    [(["hello_video"]), (["hello1"]), (["hello2"])],
)
def test_transcribe(gb_with_sources_and_settings: GailBot, source_name: List[str]):
    """
    Purpose: Test trascribing a gailbot object
    """
    result = gb_with_sources_and_settings.transcribe(source_name)
    print(result)


###########################################################################
# Profile (Setting)                                                      #
###########################################################################


@pytest.mark.parametrize(
    "name, profile, overwrite, expected",
    [
        ("Whisper", "WHISPER_PROFILE", True, True),
        ("WhisperX", "WHISPERX_PROFILE", False, True),
    ],
)
def test_create_new_profile(
    gb_with_all_engines: GailBot,
    name: str,
    profile,
    overwrite: bool,
    expected: bool,
    request,
):
    """
    Purpose: Test creating a new profile
    """
    result = gb_with_all_engines.create_new_profile(
        name, request.getfixturevalue(profile), overwrite
    )
    assert result == expected


# TODO: function not in API

# @pytest.mark.parametrize(
#     "profile_name",
#     [
#         ("whisper"),
#     ],
# )
# def test_save_profile(gb_with_profile: GailBot, profile_name: str):
#     """
#     Purpose: Test saving a profile
#     """
#     result = gb_with_profile.save_profile(profile_name)
#     assert result != False


@pytest.mark.parametrize(
    "profile_name, expected",
    [
        ("whisper", "WHISPER_PROFILE"),
    ],
)
def test_get_profile_dict(
    gb_with_profile: GailBot, profile_name: str, request, expected
):
    """
    Purpose: Test getting profile dict
    """
    result = gb_with_profile.get_profile_data(profile_name)
    assert result == request.getfixturevalue(expected)


# TODO: function not in API

# def test_get_all_profiles_data(gb_with_profile: GailBot):
#     result = gb_with_profile.get_all_profiles_data()
#     print(result)


def test_all_profile_names(gb_with_profile: GailBot):
    result = gb_with_profile.all_profile_names()
    print(result)


@pytest.mark.parametrize(
    "old_name, new_name, expected",
    [
        ("whisper", "whisper_2", True),
        ("nonexistant_profile", "nothing", False),
    ],
)
def test_rename_profile(gb_with_profile, old_name, new_name, expected):
    print(gb_with_profile.all_profile_names())
    result = gb_with_profile.rename_profile(old_name, new_name)
    if expected == True:
        gb_with_profile.rename_profile(new_name, old_name)
    assert result == expected


@pytest.mark.parametrize(
    "profile_name, new_profile, expected",
    [
        ("whisper", "WHISPERX_PROFILE", True),
        ("nonexistant_profile", "WHISPER_PROFILE", False),
    ],
)
def test_update_profile(gb: GailBot, profile_name: str, new_profile, expected, request):
    result = gb.update_profile(profile_name, request.getfixturevalue(new_profile))
    assert result == expected


# TODO: function not in API

# @pytest.mark.parametrize(
#     "profile_name, expected",
#     [
#         ("whisper", {"HiLabSuite": {"OutputFileManager": [] ,
#                             "SyllableRatePlugin": [ "OutputFileManager",]}}),
#         ("nonexistant_profile", False),
#     ],
# )
# def test_get_plugin_profile(gb_with_profile: GailBot, profile_name: str, expected):
#     result = gb_with_profile.get_plugin_profile(profile_name)
#     assert result == expected


@pytest.mark.parametrize(
    "profile_name, expected",
    [
        ("whisper", True),
        ("nonexistant_profile", False),
    ],
)
def test_remove_profile(gb_with_profile: GailBot, profile_name: str, expected: bool):
    result = gb_with_profile.remove_profile(profile_name)
    assert result == expected


# TODO: function not in API

# @pytest.mark.parametrize(
#     "profile_names, expected",
#     [
#         (["whisper"], True),
#         (["whisper", "nonexistant_profile"], False),
#     ],
# )
# def test_remove_multiple_profiles(
#     gb_with_profile: GailBot, profile_names: List[str], expected: bool
# ):
#     result = gb_with_profile.remove_multiple_profiles(profile_names)
#     assert result == expected


@pytest.mark.parametrize(
    "profile_name, expected",
    [
        ("whisper", True),
        ("nonexistent_profile", False),
    ],
)
def test_is_profile(gb_with_profile: GailBot, profile_name: str, expected: bool):
    result = gb_with_profile.is_profile(profile_name)
    assert result == expected


@pytest.mark.parametrize(
    "source, profile_name, expected",
    [
        ("HELLO_1", "whisper", True),
        ("HELLO_1", "nonexistent_profile", True),
    ],
)
def test_apply_profile_to_source2(
    gb_with_source: GailBot, source: str, profile_name: str, expected: bool, request
):
    result = gb_with_source.apply_profile_to_source(
        request.getfixturevalue(source), profile_name
    )
    assert result == expected


@pytest.mark.parametrize(
    "sources, profile_name, expected",
    [
        (["hello1"], "whisper", True),
        (["hello1", "hello2"], "nonexistent_profile", True),
    ],
)
def test_apply_profile_to_sources(
    gb_with_2_sources: GailBot, sources: List[str], profile_name: str, expected: bool
):
    result = gb_with_2_sources.apply_profile_to_sources(sources, profile_name)
    assert result == expected


@pytest.mark.parametrize(
    "profile_name, expected",
    [
        ("Default", True),
        ("Profile2", False),
    ],
)
def test_is_profile_in_use(gb_with_sources_and_profile, profile_name, expected):
    result = gb_with_sources_and_profile.is_profile_in_use(profile_name)
    assert result == expected


def test_default_profile_name(gb_with_2_sources: GailBot):
    result = gb_with_2_sources.default_profile_name()
    assert result == "Default"


# TODO: function not in API

# @pytest.mark.parametrize(
#     "profile_name, expected",
#     [
#         ("whisper", True),
#         ("Profile2", False),
#     ],
# )
# def test_set_default_profile(
#     gb_with_profile: GailBot, profile_name: str, expected: bool
# ):
#     result = gb_with_profile.set_default_profile(profile_name)
#     assert result == expected


##########################################################################
# Plugin Suite                                                            #
##########################################################################


@pytest.mark.parametrize(
    "plugin_sources, expected",
    [
        ("PLUGIN_SUITE", None),
    ],
)
def test_register_plugin_suite(
    gb: GailBot, plugin_sources: str, expected: List[str], request
):
    result = gb.register_plugin_suite(request.getfixturevalue(plugin_sources))
    assert result == expected


# TODO: function not in API

# @pytest.mark.parametrize(
#     "suite_name",
#     [
#         ("HiLabSuite"),
#     ],
# )
# def test_get_plugin_suite(gb_with_suite: GailBot, suite_name: str):
#     result = gb_with_suite.get_plugin_suite(suite_name)
#     assert isinstance(result, PluginSuite)


@pytest.mark.parametrize(
    "suite_name, expected",
    [
        ("HiLabSuite", True),
        ("fakeSuite", False),
    ],
)
def test_is_plugin_suite(gb_with_suite: GailBot, suite_name: str, expected: bool):
    result = gb_with_suite.is_plugin_suite(suite_name)
    assert result == expected


@pytest.mark.parametrize(
    "suite_name, expected",
    [
        ("HiLabSuite", True),
    ],
)
def test_remove_plugin_suite(gb_with_suite: GailBot, suite_name: str, expected: bool):
    result = gb_with_suite.remove_plugin_suite(suite_name)
    assert result == expected


def test_get_all_plugin_suites(gb_with_suite: GailBot):
    result = gb_with_suite.get_all_plugin_suites()
    assert list(result) == [
        "HiLabSuite",
    ]


def test_get_plugin_suite_metadata(gb_with_suite: GailBot):
    result = gb_with_suite.get_plugin_suite_metadata("HiLabSuite")
    assert result == {
        "Author": "hilab",
        "Email": "hilab.tufts.edu",
        "Version": "v.0.1a",
    }


def test_get_plugin_suite_dependency_grapha(gb_with_suite: GailBot):
    result = gb_with_suite.get_plugin_suite_dependency_graph("HiLabSuite")
    print(result)


def test_get_plugin_suite_documentation_path(gb_with_suite: GailBot):
    result = gb_with_suite.get_plugin_suite_documentation_path("HiLabSuite")
    print(result)


@pytest.mark.parametrize(
    "suite_name, expected",
    [
        ("HiLabSuite", False),
    ],
)
def test_is_suite_in_use(gb_with_suite: GailBot, suite_name: str, expected: bool):
    result = gb_with_suite.is_suite_in_use(suite_name)
    assert result == expected


@pytest.mark.parametrize(
    "suite_name, expected",
    [
        ("HiLabSuite", True),
    ],
)
def test_is_official_suite(gb_with_suite: GailBot, suite_name: str, expected: bool):
    result = gb_with_suite.is_official_suite(suite_name)
    assert result == expected


@pytest.mark.parametrize(
    "suite_name",
    [("MySuite",)],
)
def test_get_suite_path(gb_with_suite: GailBot, suite_name: str):
    result = gb_with_suite.get_suite_path(suite_name)
    print(result)


# ###########################################################################
# # Engines                                                                 #
# ###########################################################################


def test_available_engines(gb_with_all_engines: GailBot):
    result = gb_with_all_engines.available_engines()
    print(result)


@pytest.mark.parametrize(
    "name, setting, overwrite, expected",
    [
        ("WHISPER_NAME", "WHISPER_SETTING", True, True),
        ("WHISPERX_NAME", "WHISPERX_SETTING", True, True),
        ("GOOGLE_NAME", "GOOGLE_SETTING", True, True),
        ("WATSON_NAME", "WATSON_SETTING", True, True),
    ],
)
def test_add_new_engine(
    name: str, setting: Dict, overwrite: bool, expected: bool, request, gb: GailBot
):
    result = gb.add_new_engine(
        request.getfixturevalue(name), request.getfixturevalue(setting), overwrite
    )
    assert result == expected


@pytest.mark.parametrize(
    "name, expected",
    [
        ("whisper", True),
        ("engine2", False),
    ],
)
def test_remove_engine_setting(name: str, expected: bool, gb_with_all_engines: GailBot):
    result = gb_with_all_engines.remove_engine(name)
    assert result == expected


@pytest.mark.parametrize(
    "name, setting, expected",
    [
        ("WHISPER_NAME", "WHISPER_SETTING", True),
        ("WHISPERX_NAME", "WHISPERX_SETTING", True),
        ("GOOGLE_NAME", "GOOGLE_SETTING", True),
        ("WATSON_NAME", "WATSON_SETTING", True),
    ],
)
def test_update_engine_setting(
    name: str, setting: Dict, expected: bool, gb_with_all_engines: GailBot, request
):
    result = gb_with_all_engines.update_engine(
        request.getfixturevalue(name), request.getfixturevalue(setting)
    )
    assert result == expected


@pytest.mark.parametrize(
    "name, expected",
    [
        ("whisper", "WHISPER_SETTING"),
        ("engine2", False),
    ],
)
def test_get_engine_data(name: str, expected, gb_with_all_engines: GailBot, request):
    result = gb_with_all_engines.get_engine_data(name)
    if expected != False:
        assert result == request.getfixturevalue(expected)
    else:
        assert result == expected


# TODO: Function not in API
#
# @pytest.mark.parametrize(
#     "name, expected",
#     [
#         ("whisper", True),
#         ("whisperX", True),
#     ],
# )
# def test_is_engine_setting_in_use(
#     name: str, expected: bool, gb_with_sources_and_settings: GailBot
# ):
#     result = gb_with_sources_and_settings.is_engine_setting_in_use(name)
#     assert result == expected


@pytest.mark.parametrize(
    "name, expected",
    [
        ("whisper", True),
        ("engine2", False),
    ],
)
def test_is_engine(gb_with_all_engines: GailBot, name: str, expected: bool):
    result = gb_with_all_engines.is_engine(name)
    assert result == expected


def test_default_engine_name(gb_with_all_engines: GailBot):
    result = gb_with_all_engines.default_engine_name()
    print(result)



import os
import unittest

from gailbot.pluginSuiteManager.suiteLoader import PluginDirectorySuiteLoader
from gailbot.shared.utils.general import paths_in_dir, get_name, delete
from test.suiteManager import SUITE_TEST_SRC_PATH, SUITE_TEST_WORKSPACE_PATH


class TestDirectoryLoader(unittest.TestCase):
    src_root = SUITE_TEST_SRC_PATH
    suite_ws = os.path.join(SUITE_TEST_WORKSPACE_PATH, "suites")
    f = open(os.path.join(SUITE_TEST_WORKSPACE_PATH, "log.txt"), "w+")
    directory_loader = PluginDirectorySuiteLoader(suites_dir=suite_ws)

    def test_loading_test_suite(self):
        suite_dir = os.path.join(self.src_root, "DummyTestSuite")
        suite = self.directory_loader.load(suite_dir_path=suite_dir, f=self.f)
        assert len(suite) == 1
        assert suite[0].suite_name == "DummyTestSuite"

    def test_fail_load_due_to_missing_files(self):
        suite_dir = os.path.join(self.src_root, "MissingFiles")
        suite = self.directory_loader.load(suite_dir_path=suite_dir, f=self.f)
        assert len(suite) == 0

    def test_fail_load_invalid_path(self):
        suit_dir = "file-does-not-exist"
        suite = self.directory_loader.load(suit_dir, self.f)
        assert len(suite) == 0

    def test_load_hilab_suite(self):
        assert self.directory_loader.load(
            "/Users/yike/GailBot/gailbot_workspace/gailbot_data/plugin_source/suites/HiLabSuite/", self.f)

    def test_download_from_req_file(self):
        req = os.path.join(self.src_root, "req.txt")
        with open(req, "w+") as f:
            f.write("scrapy==2.11.1")
        lib_src = os.path.join(self.src_root, "req")
        os.mkdir(lib_src)
        assert self.directory_loader.download_packages(req, dest=lib_src)
        paths = paths_in_dir(lib_src)
        lib_exist = False
        for p in paths:
            if "scrapy" in get_name(p).lower():
                lib_exist = True
        delete(lib_src)
        assert lib_exist


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-02-24 10:13:41
# @Email:
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-11 00:23:34
# @Descriptions : A source will be converted to a payload object at the beginning
#                 of the transcription process.
#                 Given a list of datafiles, a payload object will implement
#                 function to transcribe and analyse the sources with the specified profile setting


import os.path
from abc import ABC, abstractmethod
from copy import deepcopy
from dataclasses import dataclass
from datetime import datetime
from typing import List

from gailbot.pluginSuiteManager.suite.gbPluginMethod import GBPluginMethods
from gailbot.profileManager import ProfileObject
from gailbot.shared.utils.general import make_dir, get_name, write_json, is_file, copy
from gailbot.shared.utils.logger import makelogger
from gailbot.sourceManager.watcher import Watcher
from gailbot.payload.result import SttTranscribeResult, AnalysisResult, FormatResult
from gailbot.workspace.directory_structure import OutputFolder, TemporaryFolder
import time

logger = makelogger("payloadObject")


@dataclass
class Progress:
    Start = "\u25B6 {name} : Start"
    Waiting = "\U0001F551  {name} : Waiting"
    Transcribing = "\U0001F50A {name} : Transcribing with engine {engine}"
    Finished = "\u2705 {name}: Completed"
    Transcribed = "\u2705 {name}: Transcribed"
    Error = "\U0001F6AB {name}: Error"
    Analyzing = "\U0001F4AC {name}: Applying plugin suite {suite}"
    Analyzed = "\u2705 {name}: Analyzed"
    Formatting = "\U0001F4C1 {name}: Formatting"


class PayloadObject(ABC):
    def __init__(
        self,
        original_source: str,
        data_files: List[str],
        temp_dir_struct: TemporaryFolder,
        output_path: str,
        profile: ProfileObject,
        watchers: List[Watcher],
    ):
        """
        Parameters
        ----------
        original_source:  The original path to the initial source object from user, source will be written to meta.json
        data_files: a list of datafiles, different payload may accept different datafiles format,
                    if it is a path to a media file, the file format will be in wav
        temp_dir_struct: stores paths to temporary directory, all paths in temp_dir should be already a directory
        output_path: path to output directory
        profile: Profile setting applied to the transcription
        watchers: A list of watcher added by user for the source to monitor the transcription progress
        """
        self.data_files = data_files
        self.original_source = original_source
        self.name = get_name(original_source)
        self.profile = profile
        # payload_id is generated by the time stamp
        self.payload_id = self.name + "_" + datetime.now().strftime("%m%d_%H_%M_%S_%f")

        self.output_dir = os.path.join(output_path, self.payload_id)
        self.temp_dir_struct = temp_dir_struct
        self.watchers = deepcopy(watchers)
        self.transcription_result: SttTranscribeResult = SttTranscribeResult(
            self.temp_dir_struct.transcribe_ws
        )
        self.analysis_result: AnalysisResult = AnalysisResult(
            self.temp_dir_struct.analysis_ws
        )
        self.format_result: FormatResult = FormatResult(self.temp_dir_struct.format_ws)
        self.profile = profile
        self.engine = profile.engine_provider.make_engine()
        self.plugin_suites = profile.plugin_suites


        self.source_dir = self.temp_dir_struct.data_copy

        # merged_audio and output_dir_struct will not be initialized until transcription starts
        self.merged_audio: str | None = None
        self.output_dir_struct: OutputFolder | None = None
        self._starting_time = 0

    @abstractmethod
    def stt_transcribe(self) -> bool:
        """
        subclass is responsible to implement this function that after it is invoked, the result of transcribing
        [self.datafiles] will be stored in [self.transcription_result]

        Returns
        -------
        return true of the transcription succeed, false otherwise
        """
        raise NotImplementedError()

    @abstractmethod
    def merge_audio(self, output) -> str:
        """
        subclass is responsible to implement this function that based on the structure of [self.data_files],
        merge all audio files in the payload to a single file stored in the [output] directory
        Parameters
        ----------
        output

        Returns
        -------
        path where the merged audio file is stored
        """
        raise NotImplementedError()

    def send_msg_to_watchers(self, content: str):
        for w in self.watchers:
            w.watch(progress=f"{self.name} : {content}")

    def analyze(self) -> bool:
        """
        perform transcription analysis using the applied plugin suite

        Returns
        -------
        return true if the analysis succeed, false otherwise

        Notes
        -------
        this function will only be invoked when stt_transcribe return true,
        """
        try:
            results = dict()
            for suite, selected in self.plugin_suites.items():

                suite_output = os.path.join(self.output_dir_struct.analysis, suite.name)
                suite_ws = os.path.join(self.temp_dir_struct.analysis_ws, suite.name)
                make_dir(suite_output)
                make_dir(suite_ws)
                for watcher in self.watchers:
                    watcher.watch(
                        Progress.Analyzing.format(name=self.name, suite=suite.name)
                    )
                result = suite(
                    base_input=None,
                    methods=GBPluginMethods(
                        work_path=suite_ws,
                        out_path=suite_output,
                        data_files=self.data_files,
                        merged_media=self.merged_audio,
                        utterances=self.transcription_result.get_data(),
                    ),
                    selected_plugins=selected,
                ) #nothing is being returned by suite.__call__ because of summer changes. 
                results[suite.name] = result
            assert self.analysis_result.save_data(results)
            return True
        except Exception as e:
            logger.error(e, exc_info=True)
            return False

    def format(self) -> bool:
        """
        Output the transcription result

        Returns
        -------

        Notes
        -------
        this function will only be invoked when both analysis and stt_transcribe return true,
        """
        try:
            assert self.analysis_result.output(self.output_dir_struct.analysis)
            assert self.transcription_result.output(self.output_dir_struct.transcribe)
            assert self.format_result.output(self.output_dir_struct.about)
            metadata = [
                {
                    "Profile Setting": {
                        "engine_name": self.profile.profile_data.engine_setting_name,
                        "suite_setting": self.profile.profile_data.plugin_suite_setting,
                    },
                    "Date": datetime.now().strftime("%Y-%m-%d %H:%M"),
                    "Source": self.original_source,
                    "Plugin Results": self.analysis_result.get_serialized_data(),
                    "time_elapsed": f"{time.process_time() - self._starting_time} seconds",
                }
            ]

            metafile = os.path.join(self.output_dir_struct.about, "meta.json")
            write_json(metafile, metadata)

            with open(
                os.path.join(
                    self.output_dir_struct.root, OutputFolder.GB_RESULT_SIGNATURE
                ),
                "w+",
            ) as f:
                f.write(f"{self.name}")
            for file in self.data_files:
                if is_file(file):
                    copy(
                        file,
                        os.path.join(
                            self.output_dir_struct.media, os.path.basename(file)
                        ),
                    )
            for suite, selected in self.plugin_suites.items():
                analysis = os.path.join(self.temp_dir_struct.analysis_ws, suite.name, "final")
                print("HIU", is_file(analysis))
                print(copy(analysis, os.path.join(self.output_dir_struct.analysis, suite.name, "final_analysis.xml"),))
            return True
        except Exception as e:
            logger.error(e, exc_info=True)
            return False

    def get_transcription_result(self):
        return self.transcription_result.get_data()

    def start_execute(self) -> bool:
        logger.info(f"Start transcribing {self.name}")
        self._starting_time = time.process_time()
        # initializing output directory and create merged audio file
        self.output_dir_struct = OutputFolder(self.output_dir)
        # since merged_audio can be used by plugin suite, it needs to be initialized before analyze phase
        self.merged_audio = self.merge_audio(self.output_dir_struct.media)
        logger.info(f"reached merged_audio")

        self.output_dir = self.output_dir_struct.root
        transcribed = self.stt_transcribe()
        logger.info(f"passed stt_transcribe")

        if transcribed:
            analyzed = self.analyze()
            print("\n\n\nAnalyzed! Results: ", analyzed)
        else:
            logger.error(f"{self.name} failed transcription")
            return False

        # if analyzed:
        for watcher in self.watchers:
            watcher.watch(Progress.Formatting.format(name=self.name))
        formatted = self.format()
        # else:
        #     logger.error(f"{self.name} failed analysis")
        #     return False

        if formatted:
            for watcher in self.watchers:
                watcher.watch(Progress.Finished.format(name=self.name))
            return True
        else:
            logger.error(f"{self.name} failed formatting")
            return False



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-08 12:43:29
# @Last Modified by:   Jason Y. Wu
# @Last Modified time: 2023-08-01 05:32:12
from typing import Dict, List
from .core import WatsonCore
from .lm import WatsonLMInterface
from .am import WatsonAMInterface
from gailbot.engineManager.engine.engine import Engine
from gailbot.shared.exception.transcribeException import TranscriptionError
from gailbot.configs import watson_config_loader
from gailbot.shared.utils.logger import makelogger
from .watsonSetting import WatsonSetting

WATSON_CONFIG = watson_config_loader()

logger = makelogger("watson")


class Watson(Engine):
    """
    An Engine that connect to IBM Watson STT, provide function to transcribe
    audio file with IBM Watson STT

    Inheritance:
        Engine
    """

    def __init__(
            self,
            setting: WatsonSetting
    ):
        self.setting = setting
        apikey = self.setting.apikey
        region = self.setting.region
        self.core = WatsonCore(apikey, region, setting)
        self.lm = WatsonLMInterface(apikey, region)
        self.am = WatsonAMInterface(apikey, region)
        self.is_transcribe_success = False

    def __str__(self):
        return WATSON_CONFIG.name

    def __repr__(self):
        """
        Returns all the configurations and additional metadata
        """
        return self.core.__repr__()

    @staticmethod
    def validate_api(apikey: str, region: str):
        return WatsonCore.valid_region_api(apikey, region)

    @property
    def supported_formats(self) -> List[str]:
        """
        A list of supported format that can be transcribe with the STT engine
        """
        return self.core.supported_formats

    @property
    def regions(self) -> Dict:
        """
        A dictionary of the supported regions and the regions url
        """
        return self.core.regions

    @property
    def defaults(self) -> Dict:
        """
        A dictionary that contains the default settings that will be
        applied to the IBM Watson STT engine
        """
        return self.core.defaults

    def transcribe(
            self,
            audio_path: str,
            payload_workspace: str,
    ) -> List[Dict]:
        try:
            utterances = self.core.transcribe(
                audio_path,
                payload_workspace,
                self.setting.base_model,
                self.setting.language_customization_id,
                self.setting.acoustic_customization_id,
                self.setting
            )
            self.is_transcribe_success = True
            return utterances
        except Exception as e:
            logger.error(e, exc_info=e)
            raise TranscriptionError(e)

    def language_customization_interface(self) -> WatsonLMInterface:
        """
        Return the watson customized language model interface
        """
        return self.lm

    def acoustic_customization_interface(self) -> WatsonAMInterface:
        """
        Return the watson customized acoustic model interface
        """
        return self.am

    def get_engine_name(self) -> str:
        """
        Return the name of the watson engine
        """
        return "watson"

    def get_supported_formats(self) -> List[str]:
        """
        Return a list of supported format that can be
        transcribed with the STT engine
        """
        return self.core.supported_formats

    def is_file_supported(self, file_path: str) -> bool:
        """
        Given a file path, return true if the file format is supported by
        the Watson STT engine

        Args:
            file_path: str
                a path to the file of which to check if supported
        """
        return self.core.is_file_supported(file_path)

    def was_transcription_successful(self) -> bool:
        """
        Return true if the transcription is finished and successful,
        false otherwise
        """
        return self.is_transcribe_success



# -*- coding: utf-8 -*-
# @Author: Vivian Li 
# @Date:   2024-02-24 14:33:17
# @Last Modified by:   Joanne Fan
# @Last Modified time: 2024-03-13 15:18:26
# @Description: Integration testing to transcribe sources through api interface
import os.path
import pytest

from gailbot import GailBot
from gailbot import ConverterType
from test.test_data import TestData


@pytest.fixture(scope="module", autouse=True)
def gb():
    gailbot = GailBot()
    return gailbot


def test_transcribe_audio(gb):
    # add single audio file
    gb.add_source(
        source_path=TestData.hello,
        output_dir=TestData.OUTPUT
    )

    result = gb.transcribe()
    assert result.success_output
    assert not (result.invalid or result.failure)


@pytest.mark.parametrize("video_file", ["hello.mp4",
                                        "hello.mov",
                                        "hello.3gp",
                                        "hello.avi",
                                        "short.mxf"])
def test_transcribe_video(gb, video_file):
    # add single video file
    gb.add_source(
        source_path=os.path.join(TestData.INPUT, video_file),
        output_dir=TestData.OUTPUT
    )

    result = gb.transcribe()
    assert result.success_output
    assert not (result.invalid or result.failure)


@pytest.mark.parametrize("audio_file", ["hello.mp3",
                                        "hello.wav",
                                        "hello.aac"])
def test_transcribe_audio(gb, audio_file):
    gb.add_source(
        source_path=os.path.join(TestData.INPUT, audio_file),
        output_dir=TestData.OUTPUT
    )

    result = gb.transcribe()
    assert result.success_output
    assert not (result.invalid or result.failure)


@pytest.mark.parametrize("video_file", ["short.mxf"])
def test_transcribe_mxf_video(gb, video_file):
    # add single video file
    gb.add_source(
        source_path=os.path.join(TestData.INPUT, video_file),
        output_dir=TestData.OUTPUT
    )

    result = gb.transcribe()
    assert result.success_output
    assert not (result.invalid or result.failure)


@pytest.mark.parametrize("con_dir", ["mix_format"])
def test_mixed_dir(gb, con_dir):
    # add single video file
    id = gb.add_source(
        source_path=os.path.join(TestData.MIX_DIR, con_dir),
        output_dir=TestData.OUTPUT
    )
    gb.change_source_converter(id, ConverterType.MixedDirectory)
    result = gb.transcribe()
    assert result.success_output
    assert not (result.invalid or result.failure)


def test_transcribe_conv_dir(gb):
    gb.add_source(
        source_path=TestData.test2ab,
        output_dir=TestData.OUTPUT
    )

    result = gb.transcribe()
    assert result.success_output
    assert not (result.invalid or result.failure)


def test_transcribed_dir(gb):
    gb.add_source(
        source_path=TestData.test2ab_transcribed,
        output_dir=TestData.OUTPUT
    )

    result = gb.transcribe()
    assert result.success_output
    assert not (result.invalid or result.failure)


def test_change_converter(gb):
    sourceId = gb.add_source(
        source_path=TestData.small_dir2,
        output_dir=TestData.OUTPUT
    )
    gb.change_source_converter(sourceId, ConverterType.MixedDirectory)
    result = gb.transcribe([sourceId])
    assert result.success_output
    assert not (result.invalid or result.failure)



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-07-17 19:21:51
# @Last Modified by:   Vivian Li 
# @Last Modified time: 2024-02-13 19:48:09
import os
import sys

import pip
from cryptography.fernet import Fernet
from typing import List, Optional, TextIO

from .pluginSuiteLoader import PluginSuiteLoader
from gailbot.shared.utils.logger import makelogger
from gailbot.shared.utils.general import (
    get_name,
    copy,
    read_toml,
    is_directory,
    delete,
)
from gailbot.config_backend import PROJECT_ROOT
from gailbot.configs import PLUGIN_CONFIG
from pydantic import ValidationError

from ..error.errorMessage import SUITE_REGISTER_MSG
from ..suite.pluginData import ConfModel
from ..suite.suite import PluginSuite

logger = makelogger("plugin directory loader")


class PluginDirectorySuiteLoader(PluginSuiteLoader):
    """load the plugin suite from a directory that contains all source
    script implementing the plugins, and a toml file that stores
    configuration information to load the plugin
    """

    def __init__(
        self,
        suites_dir: str,
    ):
        """initialize a plugin directory loader

        Args:
            suites_dir (str): the path to the directory that stores all the
                              copies of plugins will be stored and managed
                              by plugin manager
        """
        self.suites_dir = suites_dir

    def load(self, suite_dir_path: str, f: TextIO) -> List[PluginSuite]:
        """load a plugin suite directory in"""
        missing_file = False
        if (not isinstance(suite_dir_path, str)) or (not is_directory(suite_dir_path)):
            return []

        suite_dir_name = get_name(suite_dir_path)
        tgt_path = os.path.join(self.suites_dir, get_name(suite_dir_path))
        config = None
        requirement = None
        official = None

        # search for the requirements and config file
        for root, dirs, files in os.walk(suite_dir_path):
            if PLUGIN_CONFIG.REQUIREMENT in files:
                requirement = os.path.join(root, PLUGIN_CONFIG.REQUIREMENT)
            if PLUGIN_CONFIG.CONFIG in files:
                config = os.path.join(root, PLUGIN_CONFIG.CONFIG)
            if PLUGIN_CONFIG.OFFICIAL in files:
                official = os.path.join(root, PLUGIN_CONFIG.OFFICIAL)
            if config and requirement and official:
                break
        
        if not config:
            e = SUITE_REGISTER_MSG.MISSING.format(file=PLUGIN_CONFIG.CONFIG)
            logger.error(e)
            f.write(e)
            missing_file = True


        if missing_file:
            return []

        # download required package
        try:
            if requirement:
                assert self.download_packages(requirement, PROJECT_ROOT)
        except Exception as e:
            logger.error(e, exc_info=True)
            f.write(
                SUITE_REGISTER_MSG.TEMPLATE.format(
                    cause="fail to download plugin suite dependency"
                )
            )
            return []

        # make a copy of the original plugin suite

        # NOTE: when the tgt_path already exists, will remove tgt_path
        if not is_directory(tgt_path):
            copy(suite_dir_path, tgt_path)
        elif os.path.samefile(tgt_path, suite_dir_path):
            pass
        else:
            delete(tgt_path)
            copy(suite_dir_path, tgt_path)
        sys.path.append(tgt_path)

        suite = self.load_suite_from_toml(config, suite_dir_name, self.suites_dir, f)
        if suite:
            # validate
            if self.validate_official(official):
                suite.set_to_official_suite()
            return [suite]
        else:
            sys.path.pop()
            # TODO: why??? delete(tgt_path)
            return []

    def download_packages(self, req_file, dest):
        """download packages listed under req_file to dest

        Args:
            req_file(str): a string that specifies the path to requirements.txt file
            dest (str): a string to the directory where the file will be downloaded

        Returns:
            return true if the package is downloaded successfully, else return false
        """
        if hasattr(pip, "main"):
            pip.main(["install", "-t", str(dest), "-r", req_file])
            return True
        else:
            return False

    def validate_official(self, file):
        """
        Returns:
            bool: return true if the key matches with the official gailbot plugin
        """
        if not file:
            return False
        try:
            with open(file, "r") as f:
                key = f.read()
            fernet = Fernet(PLUGIN_CONFIG.OFFICIAL_ENKEY)

            decrypt = fernet.decrypt(key)

            if decrypt == PLUGIN_CONFIG.OFFICIAL_KEY:
                return True
            else:
                return False
        except Exception as e:
            logger.error(e, exc_info=e)
            return False

    def load_suite_from_toml(
        self, conf_path: str, suite_name: str, suites_directory: str, f: TextIO
    ) -> Optional[PluginSuite]:
        """given the path to configuration file of one plugin suite, and
             the suites directory that stores all plugin suites ,
             import the plugin suite described in the configuration file

        Returns:
            PluginSuite:
        """
        conf = read_toml(conf_path)
        try:
            conf_model = ConfModel(**conf)
        except ValidationError as e:
            f.write(
                SUITE_REGISTER_MSG.TEMPLATE.format(
                    cause=f" failed to validate configuration file structure due to {e}"
                )
            )
            logger.error(
                f"plugin suite config.toml does not follow the file structure."
            )
            return None
        if conf_model.suite.name != suite_name:
            e = f"suite directory and configuration file should use the same suite name."
            f" {conf_model.suite.name} and {suite_name} are different"
            f.write(SUITE_REGISTER_MSG.TEMPLATE.format(cause=e))

        # TODO: download all plugins that are part of the suite to general plugins folder HERE
        try:
            print("about to return plugin suite from load toml\n")
            return PluginSuite(conf_model, suites_directory)
        except Exception as e:
            print("error here 201 directory loder\n")
            logger.error(e, exc_info=True)
            f.write(SUITE_REGISTER_MSG.TEMPLATE.format(cause=e))
            return None



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-07-31 16:21:27
# @Last Modified by:   Jason Y. Wu
# @Last Modified time: 2023-08-01 05:37:57
from dataclasses import dataclass
from datetime import date, time


@dataclass
class ProcessingStats:
    """
    Defines a class to contain stats about each processing item
    """

    date: date = date.today().strftime("%m/%d/%y")
    start_time: time | float = None
    end_time: time | float = None
    elapsed_time_sec: float = None



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-08-06 13:32:05
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-09 18:04:17
import os
import torch
from typing import List, Dict

from threading import Lock
import whisperx

from gailbot.shared.utils.logger import makelogger


from gailbot.shared.utils.media import MediaHandler

from gailbot.shared.utils.general import get_extension
from gailbot.engineManager.engine.engine import Engine
from gailbot.engineManager.engine.whisperX.whisperXSetting import WhisperXSetting

logger = makelogger("whisperX")


class WhisperX(Engine):
    model_pool = dict()
    model_pool_lock = Lock()
    batch_size = 4
    device = "cpu"

    def __init__(self, setting: WhisperXSetting):
        super().__init__()
        self.setting = setting
        self._successful = False
        self.device = self.get_device()
        self.compute_type = self.get_compute_type()

    def __str__(self):
        return "whisperX"

    def __repr__(self):
        """
        Returns all the configurations and additional metadata
        """
        return "whisperX"

    def get_device(self):
        return 'cuda' if torch.cuda.is_available() else 'cpu'

    def get_compute_type(self):
        if self.device == 'cuda':
            device_properties = torch.cuda.get_device_properties(torch.cuda.current_device())
            if device_properties.major >= 7:  
                return 'float16'
        return 'float32'

    def load_model(
        self,
        language: str
    ):
        with self.model_pool_lock:
            if language in self.model_pool:
                return self.model_pool[language]
            else:
                asr_options = { 
                    "max_new_tokens": 500, 
                    "clip_timestamps": True, 
                    "hallucination_silence_threshold": 0.5,
                    "temperatures": self.setting.temperature, 
                    "initial_prompt": self.setting.initial_prompt,
                    "condition_on_previous_text": self.setting.condition_on_previous_text,
                    "no_speech_threshold": self.setting.no_speech_threshold
                }
                model = whisperx.load_model(
                    "base", device= self.device, language=language, compute_type = "int8", asr_options= asr_options, vad_options={"vad_onset": self.setting.vad_onset, "vad_offset": self.setting.vad_offset}
                )
                self.model_pool[language] = model
                return model

    def transcribe(self, audio_path, payload_workspace) -> List[Dict[str, str]]:
        """
        Use the engine to transcribe an item
        """
        audio = whisperx.load_audio(audio_path)
        model = self.load_model(self.setting.language)
        result = model.transcribe(
            audio, batch_size=self.batch_size, language=self.setting.language, chunk_size= self.setting.chunk_size
        )
        try:
            model_a, metadata = whisperx.load_align_model(
                language_code=result["language"], device=self.device
            )
        except Exception as e:
            logger.error(e, exc_info=e)

        print("aligned  model")
        result = whisperx.align(
            result["segments"],
            model_a,
            metadata,
            audio,
            self.device,
            return_char_alignments=False,
            interpolate_method= self.setting.interpolate_method
        )
        res = []
        segments = result["segments"]
        nxt_timestamp = 0
        for segment in segments:
            for word in segment["words"]:
                res.append(
                    {
                        "start": word.get("start", nxt_timestamp),
                        "end": word.get("end", nxt_timestamp),
                        "text": word.get("word", "null"),
                        "speaker": "0",
                    }
                )
                nxt_timestamp = word.get("end", nxt_timestamp) + 0.00001
        self._successful = True
        return res

    def was_transcription_successful(self) -> bool:
        """
        Return true if the transcription is successful
        """
        return self._successful

    def get_engine_name(self) -> str:
        """
        Obtain the name of the current engine.
        """
        return "whisperX"

    def get_supported_formats(self) -> List[str]:
        """
        Obtain a list of audio file formats that are supported.
        """
        return ["wav"]

    def is_file_supported(self, filepath: str) -> bool:
        """
        Determine if the given file is supported by the engine.
        """
        return get_extension(filepath) in self.get_supported_formats()



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-07-17 19:21:51
# @Last Modified by:   Lakshita Jain
# @Last Modified time: 2023-12-07 12:45:12
from dataclasses import dataclass
import os
from dict_to_dataclass import field_from_dict, DataclassFromDict
from typing import Dict, List
import toml


@dataclass
class PluginDefault(DataclassFromDict):
    suite: str = field_from_dict()
    apply_plugins: List[str] = field_from_dict()


@dataclass
class EngineDefault(DataclassFromDict):
    name: str = field_from_dict()
    whisper: Dict = field_from_dict()
    watson: Dict = field_from_dict()


@dataclass
class Default(DataclassFromDict):
    def __init__(self, plugin: PluginDefault, engine: EngineDefault) -> None:
        self.plugin: PluginDefault = plugin
        self.engine: EngineDefault = engine


def load_default_config(path: str):
    d = toml.load(path)
    engine = EngineDefault.from_dict(d["engine"])
    plugin = PluginDefault.from_dict(d["plugins"])
    return Default(plugin, engine)



from gailbot.payload.payloadObject.payloadObject import PayloadObject
import os.path
from abc import ABC, abstractmethod
from copy import deepcopy
from dataclasses import dataclass
from datetime import datetime
from typing import List
from gailbot.pluginSuiteManager.suite.gbPluginMethod import GBPluginMethods
from gailbot.profileManager import ProfileObject
from gailbot.shared.utils.general import make_dir, get_name, write_json, is_file, copy
from gailbot.shared.utils.logger import makelogger
from gailbot.sourceManager.watcher import Watcher
from gailbot.payload.result import SttTranscribeResult, AnalysisResult, FormatResult
from gailbot.workspace.directory_structure import OutputFolder, TemporaryFolder
import time


logger = makelogger("AnalysisTest")


@dataclass
class Progress:
    Start = "\u25B6 {name} : Start"
    Waiting = "\U0001F551  {name} : Waiting"
    Transcribing = "\U0001F50A {name} : Transcribing with engine {engine}"
    Finished = "\u2705 {name}: Completed"
    Transcribed = "\u2705 {name}: Transcribed"
    Error = "\U0001F6AB {name}: Error"
    Analyzing = "\U0001F4AC {name}: Applying plugin suite {suite}"
    Analyzed = "\u2705 {name}: Analyzed"
    Formatting = "\U0001F4C1 {name}: Formatting"



class Analysis():
    def __init__(
        self,
        payload_obj: PayloadObject
    ):
        self.original_source = payload_obj.original_source
        self.data_files = payload_obj.data_files
        self.temp_dir_struct = TemporaryFolder
        self.output_path = payload_obj.output_path
        self.profile  = payload_obj.profile
        self.watchers = payload_obj.watchers
        self.transcription_result = payload_obj.transcription_result
        self.plugin_suites = self.profile.plugin_suites
        self.output_dir_struct = OutputFolder(self.output_dir)
        self.merged_audio = payload_obj.merged_audio


    def analyze(self):
        """
        perform transcription analysis using the applied plugin suite

        Returns
        -------
        return true if the analysis succeed, false otherwise

        Notes
        -------
        this function will only be invoked when stt_transcribe return true,
        """
        try:
            results = dict()
            for suite, selected in self.plugin_suites.items():
                suite_output = os.path.join(self.output_dir_struct.analysis, suite.name)
                suite_ws = os.path.join(self.temp_dir_struct.analysis_ws, suite.name)
                make_dir(suite_output)
                make_dir(suite_ws)
                for watcher in self.watchers:
                    watcher.watch(
                        Progress.Analyzing.format(name=self.name, suite=suite.name)
                    )
                result = suite(
                    base_input=None,
                    methods=GBPluginMethods(
                        work_path=suite_ws,
                        out_path=suite_output,
                        data_files=self.data_files,
                        merged_media=self.merged_audio,
                        utterances=self.transcription_result.get_data(),
                    ),
                    selected_plugins=selected,
                )
                results[suite.name] = result
            # assert self.analysis_result.save_data(results)
            return results
        except Exception as e:
            logger.error(e, exc_info=True)
            return None



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-02-07 10:27:43
# @Last Modified by:   Muhammad Umair
# @Last Modified time: 2023-02-07 18:11:18
from dataclasses import dataclass
from dict_to_dataclass import field_from_dict, DataclassFromDict
from typing import Any, Union
import toml


@dataclass
class WhisperTranscribeConfig(DataclassFromDict):
    remove_punctuation_from_words : bool = field_from_dict()
    compute_word_confidence : bool = field_from_dict()
    include_punctuation_in_confidence : bool = field_from_dict()
    refine_whisper_precision : bool = field_from_dict()
    min_word_duration : float = field_from_dict()
    plot_word_alignment : bool = field_from_dict()
    naive_approach : bool = field_from_dict()
    compression_ratio_threshold : float = field_from_dict()
    logprob_threshold : Union[float, int] = field_from_dict()
    no_speech_threshold : float = field_from_dict()
    condition_on_previous_text : bool = field_from_dict()
    verbose : bool = field_from_dict()
    best_of : int = field_from_dict(default=None)
    beam_size : int = field_from_dict(default=None)
    patience : float = field_from_dict(default=None)
    length_penalty : float  = field_from_dict(default=None)
    fp16 : Any = field_from_dict(default=None)

@dataclass
class WhisperDiarizationConfig(DataclassFromDict):
    HF_auth_token : str =  field_from_dict()
    HF_diarization_config_repo_id : str = field_from_dict()
    HF_diarization_model_repo_id : str = field_from_dict()
    config_filename : str = field_from_dict()
    model_filename : str = field_from_dict()

@dataclass
class WhisperConfig(DataclassFromDict):
    engine_name : str = field_from_dict()
    model_name : str =  field_from_dict()
    transcribe_configs : WhisperTranscribeConfig = field_from_dict()
    diarization_configs : WhisperDiarizationConfig = field_from_dict()

def load_whisper_config(path: str):
    """Loads data from the Watson engine configuration

    Args:
        path (str) : path to the toml file to load
    """
    d = toml.load(path)
    return WhisperConfig.from_dict(d["whisper"])


# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-31 15:31:51
# @Last Modified by:   Jason Y. Wu
# @Last Modified time: 2023-08-07 21:47:33

from dataclasses import dataclass
from dict_to_dataclass import field_from_dict, DataclassFromDict
import toml
import os
import logging

PROJECT_ROOT = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
CONFIG_ROOT = os.path.join(PROJECT_ROOT, "config_backend")

logging.info(CONFIG_ROOT)


@dataclass
class ConfigPath(DataclassFromDict):
    """
    Loads paths to engine configuration files
    """
    watson: str = field_from_dict()
    whisper: str = field_from_dict()
    google: str = field_from_dict()
    log: str = field_from_dict()
    ws_root: str = field_from_dict()
    paths_config: str = field_from_dict()
    services: str = field_from_dict()
    default_setting: str = field_from_dict()
    valid_plugin: str = field_from_dict()
    format_md: str = field_from_dict()


path_dict = toml.load(os.path.join(CONFIG_ROOT, "paths.toml"))
PATH = ConfigPath.from_dict(path_dict["paths"])



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-03-13 18:20:30
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-06 14:31:16
import os

import pytest

from gailbot import GailBot
from test.test_data import TestData


@pytest.fixture(scope="module", autouse=True)
def gb():
    gailbot = GailBot()
    return gailbot


@pytest.mark.parametrize("audio_file", ["all/60sec.mp3"])
def test_transcribe_audio(gb, audio_file):
    gb.add_source(
        source_path=os.path.join(TestData.INPUT, audio_file), output_dir=TestData.OUTPUT
    )

    result = gb.transcribe()
    assert result.success_output
    assert not (result.invalid or result.failure)


@pytest.mark.parametrize("video_file", ["short.mxf"])
def test_transcribe_mxf_video(gb, video_file):
    # add single video file
    gb.add_source(
        source_path=os.path.join(TestData.INPUT, video_file), output_dir=TestData.OUTPUT
    )

    result = gb.transcribe()
    assert result.success_output
    assert not (result.invalid or result.failure)


def test_transcribed_2ab():
    gb = GailBot()
    gb.add_source(
        source_path=TestData.test2ab_transcribed,
        output_dir=TestData.OUTPUT,
    )
    result = gb.transcribe()
    assert not result.failure



# -*- coding: utf-8 -*-
# @Author: Erin & Joanne
# @Date:   2024-02-11 13:26:17
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-02-18 14:53:37
# @Description: Provides method retrieve metadata from aws buckets and objects.

import boto3
import os
from gailbot.pluginSuiteManager.APIConsumer import APIConsumer
from threading import Lock

class S3BucketManager:
    _instance = None
    _lock = Lock()

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super(S3BucketManager, cls).__new__(cls)
                    api_consumer = APIConsumer.get_instance()
                    response = api_consumer.fetch_secrets()
                    cls._instance.aws_api_key = response['aws_api_key']
                    cls._instance.aws_api_id = response['aws_api_id']

                    print(f"aws_api_key: {cls._instance.aws_api_key}")
                    print(f"aws_api_id: {cls._instance.aws_api_id}")
        return cls._instance
        
    @classmethod
    def get_instance(cls):
        # if it doesn't exist yet, calling cls() will run __new__
        if cls._instance is None:
            cls._instance = cls()
        return cls._instance

    @classmethod
    def reset_instance(cls):
        with cls._lock:
            cls._instance = None

    # Retrieve and return the version of the bucket
    # TODO: exceptions are not catched here. Caller expected to catch exceptions
    def get_remote_version(self, bucket_name, object_name) -> str:
        """
        If bucket_name and object_name identifies an existing object in aws s3 bucket, and the
        object has s3 remote metadata for version, returns the updated version of the object

        Parameters
        ----------
        bucket_name
        object_name

        Returns
        -------
        str

        Raises
        -------

        """
        s3 = boto3.client(
            "s3",
            aws_access_key_id=self.aws_api_id,
            aws_secret_access_key=self.aws_api_key,
        )

        s3_object = s3.head_object(Bucket=bucket_name, Key=object_name)
        object_metadata = s3_object["Metadata"]

        return object_metadata["version"]
    
    def get_s3(self):
        s3 = boto3.client(
            "s3",
            aws_access_key_id=self.aws_api_id,
            aws_secret_access_key=self.aws_api_key,
        )

        return s3
    
    def get_r3(self):
        r3 = boto3.resource(
            "s3",             
            aws_access_key_id=self.aws_api_id,
            aws_secret_access_key=self.aws_api_key,
        )

        return r3


# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-01-29 13:26:17
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-02-06 16:03:16
# @Description: This is initially an abstract class that declare a set of function
# needs to be implemented for any  plugin method, currently, there is only one
# GBPluginMethods
import os
from dataclasses import dataclass
from typing import List, Dict, Union, Any

from gailbot.engineManager.engine.engine import UttDict
from gailbot.shared.utils.general import write_csv, write_toml, write_yaml, write_json, write_txt, get_name
from gailbot.shared.utils.logger import makelogger

logger = makelogger("Plugin Method")


class Methods:
    """
    Methods that will be passed to a plugin.
    These can be custom defined and may be useful as
    a wrapper around objects
    that may want to be passed to a plugin.
    """

    def __init__(self):
        pass


class UttObj:
    def __init__(self, start: float | str, end: float | str, speaker, text):
        self.start = float(start)
        self.end = float(end)
        self.speaker = speaker
        self.text = text


@dataclass(init=True)
class GBPluginMethods(Methods):
    work_path: str
    out_path: str
    data_files: List[str]
    merged_media: str
    utterances: Dict[str, List[UttDict]]

    @property
    def format_to_out_fun(self):
        return {
            "csv": write_csv,
            "toml": write_toml,
            "yaml": write_yaml,
            "json": write_json,
            "txt": write_txt,
        }

    @property
    def temp_work_path(self):
        return self.work_path

    @property
    def output_path(self):
        return self.out_path

    @property
    def filenames(self):
        return [get_name(file) for file in self.data_files]

    def get_utterance_objects(self) -> Dict[str, List[UttObj]]:
        """
        Access and return the utterance data as utterance object
        """
        res = dict()
        for key, uttlist in self.utterances.items():
            newlist = list()
            for utt in uttlist:
                # Convert speaker to string, handle None case

                utt["speaker"] = (
                    str(utt["speaker"]) if utt["speaker"] is not None else "speaker_00"
                )
                newlist.append(UttObj(**utt))
            res[key] = newlist
        return res

    def save_item(
            self,
            data: Union[Dict[str, Any], List],
            name: str,
            temporary: bool = True,
            output_format: str = "json",
            fun: callable = None,
            kwargs=None,
    ) -> bool:
        """
        function provided for the plugin to save file

        Args:
            data (Union[Dict[str, Any], List]): the data that will be outputed
            name (str): the name of the output file
            temporary (bool, optional): if true, the file will be stored in
                                        temporary folder and discarded once the
                                        analysis process finishes. Defaults to True.
            output_format (str, optional): the format of the output file. Defaults to "json".
            fun (callable, optional): user defined function to write the
                                      output. Defaults to None.
            kwargs (dict, optional): user defined key word arguments that will
                                    be passed into user defined function.
                                    Defaults to None.

        Returns:
            bool: return true if the plugin is registered successfully , false otherwise
        """
        path = (
            os.path.join(self.work_path, name + "." + output_format)
            if temporary
            else os.path.join(self.out_path, name + "." + output_format)
        )
        if fun:
            try:
                fun(path, data, **kwargs)
                return True
            except Exception as e:
                logger.error(e, exc_info=e)
                return False
        if output_format not in self.format_to_out_fun:
            logger.error("the output format is not supported")
            return False
        try:
            self.format_to_out_fun[output_format](path, data)
            return True
        except Exception as e:
            logger.error(e, exc_info=e)
            return False



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-31 15:29:08
# @Last Modified by:   Muhammad Umair
# @Last Modified time: 2023-01-31 15:37:25

import os
import userpaths
from ..confs.paths import PATH, CONFIG_ROOT, PROJECT_ROOT
from .core.engines.watson import load_watson_config
from .core.engines.google import load_google_config
from .core.engines.whisper import load_whisper_config
from .core.setting.defaults import load_default_config
from .core.util.logger import load_log_config
from .config.ws_config import load_workspace_config
from .plugin import PLUGIN_CONFIG, load_valid_structure
from .services import load_service_config, load_default_setting
from .config.ws_config import TemporaryFolder, OutputFolder


def watson_config_loader(): return load_watson_config(
    os.path.join(CONFIG_ROOT, PATH.watson))


def google_config_loader(): return load_google_config(
    os.path.join(CONFIG_ROOT, PATH.google))


def whisper_config_loader(): return load_whisper_config(
    os.path.join(CONFIG_ROOT, PATH.whisper)
)


def service_config_loader(): return load_service_config(
    os.path.join(CONFIG_ROOT, PATH.services)
)


def log_config_loader(): return load_log_config(
    os.path.join(CONFIG_ROOT, PATH.log)
)


def workspace_config_loader(): return load_workspace_config(
    os.path.join(CONFIG_ROOT, PATH.paths_config),
    os.path.join(userpaths.get_profile(), "Gailbot")
)


def default_setting_loader(): return load_default_setting(
    os.path.join(CONFIG_ROOT, PATH.default_setting)
)


def get_plugin_structure_config():
    return load_valid_structure(os.path.join(CONFIG_ROOT, PATH.valid_plugin))


def get_format_md_path():
    return os.path.join(CONFIG_ROOT, PATH.format_md)



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-01-25 18:14:59
# @Last Modified by:   Vivian Li 
# @Last Modified time: 2024-05-03 17:16:02
import os
import platform
from typing import Dict
from gailbot.shared.exception.serviceException import (
    EngineNotFound,
    DuplicateProfile,
    RemoveDefaultEngine,
)
from gailbot.engineManager.engine.google.googleSetting import GoogleSetting
from gailbot.engineManager.engine.watson.watsonSetting import WatsonSetting
from gailbot.engineManager.engine.whisperX.whisperXSetting import WhisperXSetting
from gailbot.engineManager.engine.whisper.whisperSetting import WhisperSetting
from gailbot.engineManager.engine.deepgram.deepgramSetting import DeepgramSetting
from gailbot.engineManager.engine.engineProvider import EngineProvider
from gailbot.engineManager.engine.google.googleProvider import GoogleProvider
from gailbot.engineManager.engine.watson.watsonProvider import WatsonProvider
from gailbot.engineManager.engine.whisper.whisperProvider import WhisperProvider
from gailbot.engineManager.engine.whisperX.whisperXProvider import WhisperXProvider
from gailbot.engineManager.engine.deepgram.deepgramProvider import DeepgramProvider
from gailbot.setting_interface.formItem import FormItem
from gailbot.shared.utils.logger import makelogger
from gailbot.shared.utils.general import (
    is_file,
    delete,
    write_toml,
    paths_in_dir,
    get_name,
    read_toml,
    make_dir,
)


class EngineManager:
    engine_provider = {
        "google": GoogleProvider,
        "watson": WatsonProvider,
        "deepgram": DeepgramProvider,
        "whisper": WhisperProvider,
        "whisperX": WhisperXProvider,
    }
    default_name = "Default"
    default_data = {"engine": "whisperX", "language": "en"}
    logger = makelogger("engine-manager")

    def __init__(self, workspace: str):
        self.workspace = workspace
        if not os.path.isdir(self.workspace):
            make_dir(self.workspace)
        self._engine_providers: Dict[str, EngineProvider] = dict()
        try:
            self._load_existing_setting()
            self._load_default_setting()
        except Exception as e:
            self.logger.error(e, exc_info=True)

    def _load_default_setting(self):
        if not self.is_engine(self.default_name):
            self.add_engine(self.default_name, self.default_data)

    def _load_existing_setting(self):
        files = paths_in_dir(self.workspace, ["toml"])
        for file in files:
            name = get_name(file)
            data = read_toml(file)
            if not self._load_engine(name, data):
                # remove the file if engine is not loaded successfully
                delete(file)

    def _load_engine(self, name: str, data: Dict[str, str]) -> EngineProvider | None:
        eng_prov = EngineManager.engine_provider[data["engine"]]
        engine = eng_prov.load(name, data)
        if engine:
            self._engine_providers[name] = engine
            return engine
        self.logger.error("not a valid engine")
        return None

    def is_engine(self, name: str) -> bool:
        return name in self._engine_providers

    def add_engine(self, name: str, data: Dict[str, str]) -> bool:
        if self.is_engine(name):
            raise DuplicateProfile(name)
        loaded_engine = self._load_engine(name, data)
        if loaded_engine:
            path = os.path.join(self.workspace, f"{name}.toml")
            if is_file(path):
                delete(path)
            write_toml(path=path, data=dict(loaded_engine.data))
            return True
        else:
            return False

    def remove_engine(self, name: str):
        if name not in self._engine_providers:
            raise EngineNotFound(name)
        if name == self.default_name:
            raise RemoveDefaultEngine()
        else:
            path = os.path.join(self.workspace, f"{name}.toml")
            if is_file(path):
                delete(path)
            del self._engine_providers[name]

    def update_engine(self, name: str, new_data: Dict[str, str]) -> bool:
        if name not in self._engine_providers:
            raise EngineNotFound(name)
        else:
            path = os.path.join(self.workspace, f"{name}.toml")
            if is_file(path):
                delete(path)
            del self._engine_providers[name]
        return self.add_engine(name, new_data)

    def get_engine_provider(self, name: str) -> EngineProvider:
        if not self.is_engine(name):
            raise EngineNotFound(name)
        else:
            return self._engine_providers[name]

    def get_engine_setting(self, name) -> Dict[str, str]:
        return self.get_engine_provider(name).engine_data()

    @property
    def available_engines(self):
        return list(self._engine_providers.keys())
    
    
def get_engine_formItem() -> Dict[str, FormItem]:
   
    return {"google": GoogleSetting.get_setting_config(),
            "watson": WatsonSetting.get_setting_config(),
            "deepgram": DeepgramSetting.get_setting_config(),
            "whisperX": WhisperXSetting.get_setting_config(),
            "whisper": WhisperSetting.get_setting_config(),}



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-07-17 19:21:51
# @Last Modified by:   Jason Y. Wu
# @Last Modified time: 2023-07-24 14:08:12
import os
from dataclasses import dataclass
from dict_to_dataclass import field_from_dict, DataclassFromDict
import toml
from typing import Dict


@dataclass
class OutputFolder(DataclassFromDict):
    root: str = field_from_dict()
    transcribe_result: str = field_from_dict()
    analysis_result: str = field_from_dict()
    metadata: str = field_from_dict()
    media_file: str = field_from_dict()
    format_result: str = field_from_dict()


@dataclass
class TemporaryFolder(DataclassFromDict):
    root: str = field_from_dict()
    transcribe_ws: str = field_from_dict()
    format_ws: str = field_from_dict()
    analysis_ws: str = field_from_dict()
    data_copy: str = field_from_dict()


@dataclass
class FileExtensions(DataclassFromDict):
    temp: str = field_from_dict()
    output: str = field_from_dict()


@dataclass
class EngineWS:
    def __init__(self, ws_root: str, path_dict: Dict[str, str]) -> None:
        self.whisper = os.path.join(ws_root, path_dict["whisper"])
        self.google = os.path.join(ws_root, path_dict["google"])
        self.watson = os.path.join(ws_root, path_dict["watson"])
        self.google_api = os.path.join(ws_root, path_dict["google_api"])


@dataclass
class GailBotData:
    def __init__(self, ws_root: str, path_dict: Dict[str, str]) -> None:
        self.root: str = path_dict["root"]
        self.setting_src: str = os.path.join(
            ws_root, self.root, path_dict["setting_src"]
        )
        self.plugin_src: str = os.path.join(ws_root, self.root, path_dict["plugin_src"])
        self.engine_src: str = os.path.join(
            ws_root, self.root, path_dict["engine_setting"]
        )
        self.root: str = os.path.join(ws_root, self.root)


@dataclass
class WorkSpaceConfig:
    def __init__(self, config_path: str, ws_root: str) -> None:
        d = toml.load(config_path)
        self._output_d: Dict = d["output"]
        self._workspace_d: Dict = d["workspace"]
        self._extension_d: Dict = d["file_extensions"]
        self._ws_root = ws_root
        # public path data
        self.workspace_root = os.path.join(self._ws_root, self._workspace_d["ws_root"])
        self.tempspace_root = os.path.join(
            self.workspace_root, self._workspace_d["temporary"]["root"]
        )
        self.gailbot_data: GailBotData = GailBotData(
            self.workspace_root, self._workspace_d["gailbot_data"]
        )
        self.file_extension: FileExtensions = FileExtensions.from_dict(
            self._extension_d
        )
        self.engine_ws: EngineWS = EngineWS(
            self.workspace_root, self._workspace_d["engine"]
        )

    def get_temp_space(self, name: str) -> TemporaryFolder:
        """ Given a name of the source, return a dataclass object that stores
            the temporary directory structures of source, including the
            full paths to every subdirectory in the temporary directory

        Args:
            name (str): the name of the source

        Returns:
            TemporaryFolder: a dataclass object that stores the full paths
            of every subdirectories within the temporary folders for a
            particular source
        """
        temp_dir: Dict[str, str] = self._workspace_d["temporary"].copy()
        for key, value in temp_dir.items():
            temp_dir[key] = os.path.join(self.tempspace_root, name, value)
        temp_dir["root"] = os.path.join(self.tempspace_root, name)
        return TemporaryFolder.from_dict(temp_dir)

    def get_output_space(self, root) -> OutputFolder:
        """ Given a name of the source,  and the user selected output directory
            root, return a dataclass object that stores
            the output directory structures of source, including the
            full paths to every subdirectory in the output directory

        Args:
            root (str): the root the directory of the output
            name (str): the name of the source

        Returns:
            OutputFolder: a dataclass object that stores the full paths
            of every subdirectories within the output folders for a
            particular source
        """
        new_output_dir = self._output_d.copy()
        for key, value in new_output_dir.items():
            new_output_dir[key] = os.path.join(root, value)
        new_output_dir["root"] = root
        return OutputFolder.from_dict(new_output_dir)

    def get_output_structure(self) -> OutputFolder:
        return OutputFolder.from_dict(self._output_d)


def load_workspace_config(config_path, ws_root) -> WorkSpaceConfig:
    """public function that load the workspace data and return it"""
    return WorkSpaceConfig(config_path, ws_root)



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-31 15:37:50
# @Last Modified by:   Vivian Li
# @Description: provide function to load dataclasses that stores gailbot
# configuration data, which are parsed form raw toml file stored in
# ./config_backend

from .interfaces import (
    watson_config_loader,
    google_config_loader,
    whisper_config_loader,
    log_config_loader,
    workspace_config_loader,
    service_config_loader,
    default_setting_loader,
    PATH,
    PROJECT_ROOT,
    TemporaryFolder,
    OutputFolder,
    PLUGIN_CONFIG,
    get_format_md_path,
)



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-06 14:55:34
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-09 23:46:44


__version__ = "0.1.7"

## must export Plugin, Methods and GBPluginMethods
from .api import GailBot
from gailbot.shared.exception.serviceException import (
    SourceAddError,
    SourceNotFound,
    FailPluginSuiteRegister,
    PluginSuiteNotFound,
    ProfileNotFound,
    RemoveInUseError,
    EngineNotFound,
    DuplicateProfile,
    DuplicateEngine,
    DuplicatePlugin,
)
from gailbot.sourceManager.sourceObject import SourceObject
from gailbot.pluginSuiteManager.suite.plugin import Plugin
from gailbot.pluginSuiteManager.suite.gbPluginMethod import GBPluginMethods, Methods
from gailbot.shared.exception.gbException import GBException
from gailbot.profileManager.profileManager import ProfileSetting
from gailbot.payload.payloadConverter import ConverterType



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-08 12:52:37
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-02 17:47:48
# @Description: Pipeline takes in a dependencies map, and implement the logic
# of running inter-dependent components in parallel while maintaining the dependency
# order

from typing import List, Dict, Any
from dataclasses import dataclass

from .component import Component, ComponentState, ComponentResult
from gailbot.shared.utils.threads import ThreadPool
from gailbot.shared.utils.logger import makelogger

import networkx as nx

Failure = ComponentResult(ComponentState.FAILED, None, 0)
logger = makelogger("transcriptionPipeline")


@dataclass
class DataStream:
    data: Any = None


class Pipeline:
    """
    Defines a class for the transcriptionPipeline that runs the dependency map.
    """

    def __init__(
            self,
            dependency_map: Dict[str, List[str]],
            components: Dict[str, Component],
            num_threads: int,
    ):
        """
        Dependency map describes the execution order.
        """
        self.dependency_map = dependency_map
        self.components = components
        self.threadpool = ThreadPool(num_threads)
        self._generate_dependency_graph(dependency_map, components)

    def __repr__(self) -> str:
        """
        Accesses the transcriptionPipeline's dependency graph.

        Args:
            self

        Returns:
            String representation of the transcriptionPipeline's dependency graph.d
        """
        return str(self.get_dependency_graph())

    def __call__(
            self,
            base_input: Any,
            additional_component_kwargs: Dict = None,
            # NOTE: base_input is passed only to the first component.
    ) -> Dict[str, ComponentState]:
        """
        Execute the transcriptionPipeline by running all components in order of the
        dependency graph. This wraps data as DataStream before passing it b/w
        components. Additionally, each component receives the output of its
        dependencies.

        Args:
            base_input:
                a list of input arguments that will be passed to the first
                component of the graph

            additional_component_kwargs:
                passed as a dereferenced dictionary to each component.

        Returns:
            Dictionary containing keys mapping to the component states
            corresponding to the result of each task.

        Note:
            each component is contained in a Component class
        """
        successors = self.get_dependency_graph()
        if not additional_component_kwargs:
            additional_component_kwargs = {}
        name_to_results: Dict[str, ComponentResult] = (
            dict()
        )  # map component name to result

        while True:
            # executables is a list of Component who currently has no dependent node
            executables: List[Component] = [
                c for c, d in self.dependency_graph.in_degree if d == 0
            ]

            threadkey_to_exe: Dict[str, Component] = (
                dict()
            )  # map thread key to executable

            # exit the loop if no nodes left.
            if len(executables) == 0:
                break

            for executable in executables:
                exe_name: str = self.component_to_name[executable]
                dependency_resolved = True

                # check the result output of exe_name's dependency component
                if len(successors[exe_name]) > 0:
                    dep_outputs: Dict[str, ComponentResult] = {
                        k: name_to_results[k] for k in successors[exe_name]
                    }
                else:
                    dep_outputs = {
                        "base": ComponentResult(
                            state=ComponentState.SUCCESS, result=base_input, runtime=0
                        )
                    }

                for res in dep_outputs.values():
                    if res.state == ComponentState.FAILED:
                        name_to_results[exe_name] = Failure
                        if self.dependency_graph.has_node(executable):
                            self.dependency_graph.remove_node(executable)
                        dependency_resolved = False

                args = [dep_outputs]

                if dependency_resolved:
                    key = self.threadpool.add_task(
                        executable, args=args, kwargs=additional_component_kwargs
                    )
                    threadkey_to_exe[key] = executable

            # wait until all tasks finishes before next iteration
            self.threadpool.wait_for_all_completion(error_fun=lambda: None)

            for key, exe in threadkey_to_exe.items():
                # get the task result from the thread pool
                exe_res = self.threadpool.get_task_result(key)
                self.dependency_graph.remove_node(exe)
                name = self.component_to_name[exe]

                if exe_res and exe_res.state == ComponentState.SUCCESS:
                    # add to result if success
                    name_to_results[name] = exe_res
                else:
                    # add the failed result on failure
                    name_to_results[name] = Failure

        # Regenerate graph
        self._generate_dependency_graph(self.dependency_map, self.components)

        return {k: v.state for k, v in name_to_results.items()}

    def component_names(self) -> List[str]:
        """
        Gets names of all components in the dependency map.

        Args:
            self

        Returns:
            List of strings containing components.
        """
        return list(self.name_to_component.keys())

    def is_component(self, name: str) -> bool:
        return name in self.component_names()

    def component_parents(self, name: str) -> List[str]:
        """
        Get the component(s) that the given component is dependent on.

        Args:
            name: string containing the name of the child component.

        Returns:
            List of strings of the names of the given component's parent
            components.

            Raises exception if the given name doesn't correspond to an
            existing component.
        """
        if not self.is_component(name):
            raise Exception(f"No component named {name}")
        edges = list(self.dependency_graph.in_edges(self.name_to_component[name]))
        return [self.component_to_name[e[0]] for e in edges]

    def component_children(self, name: str) -> List[str]:
        """
        Gets component(s) that are dependent on the given component.

        Args:
            name: string containing the name of the child component.

        Returns:
            List of strings of the names of components that are dependent on the
            given component.

            Raises exception if the given name doesn't correspond to an
            existing component.
        """
        if not self.is_component(name):
            raise Exception(f"No component named {name}")
        edges = list(self.dependency_graph.out_edges(self.name_to_component[name]))
        return [self.component_to_name[e[1]] for e in edges]

    def get_dependency_graph(self) -> Dict:
        """
        Returns a map from each component to the components it is dependent on.

        Args:
            self

        Returns:
            Dictionary mapping the given component to the components it is dependent upon.
        """
        view = dict()
        for name in self.name_to_component:
            view[name] = self.component_parents(name)
        return view

    #####
    # PRIVATE METHODS
    ####

    def _does_cycle_exist(self, graph: nx.Graph) -> bool:
        """
        Determines if there are existing cycles in the given graph.

        Args:
            graph: graph in which to determine if there are cycles.

        Returns:
            True if there are any cycles in the given graph, false if not.
        """
        try:
            nx.find_cycle(graph, orientation="original")
            return True
        except:
            return False

    def _generate_dependency_graph(
            self, dependency_map: Dict[str, List[str]], components: Dict[str, Component]
    ) -> None:
        """
        Generates a dependency graph containing components from a given dictionary.
        Assumes that the dependency_map keys are in order i.e, a component will
        be seen as a key before it is seen as a dependency.

        Args:
            dependency_map: dictionary containing lists of strings to map between.
            components: dictionary containing components to insert into the newly
                created dependency graph.

        Returns:
            A graph of components mapping from the component's name to its dependencies.

            Raises exception if an element of the dependency graph does not correspond
                to a valid element.

        """
        self.dependency_graph = nx.DiGraph()
        self.name_to_component = dict()

        for name, dependencies in dependency_map.items():
            if name not in components:
                logger.warning(f"Component {name} not found in components, skipping...")
                continue

            self.dependency_graph.add_node(components[name])
            for dep in dependencies:
                if dep not in components:
                    logger.warning(
                        f"Dependency {dep} for {name} not found, skipping..."
                    )
                    continue
                self.dependency_graph.add_edge(components[dep], components[name])

            # NOTE: Cycles are not supported
            if self._does_cycle_exist(self.dependency_graph):
                raise Exception(f"Cycle found in execution logic")

            self.name_to_component[name] = components[name]

        self.component_to_name: Dict[Component, str] = {
            v: k for k, v in self.name_to_component.items()
        }



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-08-04 13:59:52
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-09 16:45:14
# @Description:
from typing import List, Dict, Any, Tuple, Optional
from gailbot.engineManager.engineManager import EngineManager
from gailbot.pluginSuiteManager.pluginSuiteManager import PluginSuiteManager
from gailbot.pluginSuiteManager.suite.pluginData import Suite
from gailbot.shared.exception.serviceException import (
    SourceAddError,
    FailPluginSuiteRegister,
    ProfileNotFound,
    RemoveInUseError,
    IncompatibleConverter,
)
from gailbot.sourceManager.watcher import Watcher
from gailbot.payload import ConvertFun
from gailbot.workspace import WorkspaceManager
from gailbot.shared.utils.logger import makelogger
from gailbot.pluginSuiteManager.APIConsumer import APIConsumer
from gailbot.pluginSuiteManager.S3BucketManager import S3BucketManager
from gailbot.profileManager.profileManager import ProfileManager, ProfileSetting
from gailbot.transcriptionPipeline.pipeline import GBPipeline, TranscriptionResult
from gailbot.sourceManager.sourceManager import SourceManager
from gailbot.configs import default_setting_loader
from tqdm import tqdm

CONFIG = default_setting_loader()
DEFAULT_SETTING_NAME = CONFIG.profile_name
logger = makelogger("gb_api")


class GailBot:
    def __init__(self):
        """
        initialize a gailbot object that provides a suite of functions
            to interact with gailbot
        """
        tqdm(disable=True, total=0)
        self.ws_manager: WorkspaceManager = WorkspaceManager()
        self._init_workspace()
        self.engine_manager = EngineManager(WorkspaceManager.engine_setting_src)
        self.source_manager = SourceManager()
        self.pipeline_service = GBPipeline
        self.transcribed = set()
        self.api = None
        self.plugin_suite_manager = None
        self.profile_manager = None

    def _init_workspace(self) -> bool:
        """
        Resets the workspace: clears the old workspace and initializes a new one.

        Returns
        -------
        Bool:
            True if the workspace is initialized successful, false otherwise.
        """
        try:
            self.ws_manager.clear_gb_temp_dir()
            self.ws_manager.init_workspace()
            return True
        except Exception as e:
            logger.error(f"failed to reset workspace due to the error {e}", exc_info=e)
            return False

    def clear_workspace(self) -> bool:
        """
        Clears current workspace.

        Returns
        -------
        Bool:
            True if the workspace is cleared, false otherwise.
        """
        try:
            self.ws_manager.clear_gb_temp_dir()
            return True
        except Exception as e:
            logger.error(f"failed to reset workspace due to the error {e}", exc_info=e)
            return False

    def reset_workspace(self) -> bool:
        """
        Reset the gailbot workspace.

        Returns
        -------
        Bool:
            True if workspace successfully reset; false otherwise.
        """
        return self.ws_manager.reset_workspace()
    
    def login(self, user_id, token):
        self.api = APIConsumer(user_id=user_id, token=token)
        S3BucketManager()
        self.plugin_suite_manager = PluginSuiteManager(WorkspaceManager.plugin_src)
        self.profile_manager = ProfileManager(
            WorkspaceManager.profile_src, self.engine_manager, self.plugin_suite_manager
        )

    def logout(self):
        print('logging out!')
        self.api.reset_instance()
        self.plugin_suite_manager = None
        self.profile_manager = None

    def add_source(self, source_path: str, output_dir: str) -> str:
        """
        Adds a given source.

        Parameters
        ----------
        source_path: str
            Full path of the source to add.
        output_dir: str
            Path to the directory where gailbot should place output files.

        Returns
        -------
        str:
            the id of the added source

        Raises
        ------
        SourceAddError
        """
        try:
            name = self.source_manager.add_source(source_path, output_dir)
            self.source_manager.apply_profile_to_source(
                name, self.profile_manager.default_profile_name
            )
            return name
        except Exception as e:
            raise SourceAddError(source_path, e)

    def add_sources(
        self, src_output_pairs: List[Tuple[str, str]]
    ) -> Dict[str, Tuple[str, str]]:
        """
        Adds a given list of sources.

        Parameters
        ----------
        src_output_pairs: List[Tuple[str, str]]
            A list of (source_path, output_directory_path) tuples.

        Returns
        -------
        Dict[str, Tuple[str, str]]:
            A dictionary mapping from source id to the corresponding
            (source_path, output_directory_path) tuple.

        Raises
        -------
        SourceAddError
        """
        res = dict()
        for source_path, output in src_output_pairs:
            source_id = self.add_source(source_path, output)
            res[source_id] = (source_path, output)
        return res

    def is_source(self, source_id: str) -> bool:
        """
        Determines if a given name corresponds to an existing source.

        Parameters
        ----------
        source_id: str
            Name of the source to look for; or path.

        Returns
        -------
        Bool:
            True if the given name corresponds to an existing source,
            false if not.
        """
        return self.source_manager.is_source(source_id)

    def remove_source(self, source_id: str):
        """
        Removes the given source.

        Parameters
        ----------
        source_id: str
            Name of the existing source to remove; without extension.

        Returns
        -------
        None

        Raises
        ------
        SourceNotFound
        """
        self.source_manager.remove_source(source_id)

    def remove_sources(self, source_ids: List[str]):
        """
        Removes the given list of named sources.

        Parameters
        ----------
        source_ids: List[str]
            List of names of the existing sources to remove

        Returns
        -------
        None

        Raises
        ------
        SourceNotFound
        """
        for name in source_ids:
            self.source_manager.remove_source(name)

    def source_profile_setting(self, source_id: str) -> ProfileSetting:
        """
        Access the contents of the profile assigned to a given source.

        Parameters
        ----------
        source_id: str
            The name of the source; or path

        Returns
        -------
        Dict:
            A dictionary storing the data of the source's profile

        Raises
        -------
        SourceNotFound
        """
        profile_name = self.source_manager.get_source_profile(source_id)
        profile = self.profile_manager.get_profile_setting(profile_name)
        return profile

    def source_profile_name(self, source_id: str) -> str:
        """
        Access the name of the profile applied to the given source.

        Parameters
        ----------
        source_id: str
            The name of the source file; without extensions.

        Returns
        -------
        str:
            The name of the profile applied to the source.

        Raises
        ---------
        SourceNotFound
        """
        return self.source_manager.get_source_profile(source_id)

    def change_source_converter(self, source_id, converter):
        """
        Parameters
        ----------
        source_id
        converter

        Returns
        -------
        true if the source [converter] can convert the source, false otherwise
        """
        if not self.source_manager.change_source_converter(source_id, converter):
            raise IncompatibleConverter(source_id, converter.value)

    def available_converters(self):
        """
        Returns
        -------
        A list of available converters' names
        """
        return self.source_manager.available_converters()

    def get_source_converter(self, source_id):
        """
        get the converter currently applied to the source identified by source_id

        Parameters
        ----------
        source_id

        Returns
        -------

        Raises
        -------
        SourceNotFoundError
        """
        return self.source_manager.get_source_converter(source_id)

    def compatible_converters(self, source_id):
        """
        get all converters that are compatible with the type of the
        source identified by source_id

        Parameters
        ----------
        source_id

        Returns
        -------

        Raises
        -------
        SourceNotFoundError
        """
        return self.source_manager.compatible_converters(source_id)

    def transcribe(self, source_ids: Optional[List[str]] = None) -> TranscriptionResult:
        """
        Given a list of source IDs, transcribe the sources.

        Parameters
        ----------
        source_ids: List[str]
            A list of source IDs to transcribe.

        Returns
        -------
        TranscriptionResult:
            The path to the successful transcriptions' outputs will be stored in [TranscriptionResult.success_output]
            The source name for the invalid and failure transcription will be stored in [TranscriptionResult.invalid]
            and [Transcription.fail]

        Raises
        ------
        """
        sources = []
        if not source_ids:
            source_ids = list(self.source_manager.un_transcribed_sources().keys())
        # get source object from source id
        for src_id in source_ids:
            sources.append(self.get_source_obj(src_id))

        print("sources is ", sources)
        # transcribe source
        result = self.pipeline_service.transcribe(sources)

        # set source to be transcribed
        for src_id in source_ids:
            self.source_manager.remove_source(src_id)
        self.ws_manager.clear_gb_temp_dir()
        return result

    def create_profile(self, name: str, setting: Dict[str, str] | ProfileSetting):
        """
        Create a new profile.

        Parameters
        ----------
        name: str
            The name of the profile.
        setting: Dict[str, str]
            The profile content.

        Raises
        ----------
        DuplicateProfile
        EngineNotFound
        """
        self.profile_manager.add_new_profile(name, setting)

    def get_profile_setting(self, name: str) -> ProfileSetting:
        """
        Given a profile name, returns the profile's content.

        Parameters
        ----------
        name: str
            Name of a profile.

        Returns
        -------
        Dict[str, Any]:
            A dictionary with the content of the profile.

        Raises
        ------
        ProfileNotFound
        """
        return self.profile_manager.get_profile_setting(name)

    def available_profiles(self) -> List[str]:
        """
        Get the names of all available profiles.

        Returns
        -------
        List[str]:
            A list of available profiles' names
        """
        if self. profile_manager:
            return self.profile_manager.available_profiles
        else:
            return []

    def update_profile(
        self, profile_name: str, new_setting: Dict[str, Any] | ProfileSetting
    ):
        """
        Updates a given profile to a newly given structure.

        Parameters
        ----------
        profile_name: str
            Name of the profile to update.
        new_setting: Dict[str, Any] | ProfileSetting
            Either dictionary or ProfileSetting object that stores the updated profile setting

        Returns
        -------
        None

        Raises
        ------
        ProfileNotFound

        """
        self.profile_manager.update_profile(profile_name, new_setting)

    def remove_profile(self, profile_name: str):
        """
        Removes the given profile.

        Parameters
        ----------
        profile_name: str
            Name of the profile to remove.

        Returns
        -------
        None

        Raises
        ------
        RemoveInUseError
        ProfileNotFound
        """
        if self.source_manager.profile_used_by_source(profile_name):
            raise RemoveInUseError(profile_name)
        else:
            self.profile_manager.remove_profile(profile_name)

    def is_profile(self, name: str) -> bool:
        """
        Determines if the given name corresponds to an existing profile.

        Parameters
        ----------
        name: str
            Name of the profile to search for.

        Returns
        -------
        Bool:
            True if given profile is an existing profile, false if not
        """
        return self.profile_manager.is_profile(name)

    def apply_profile_to_source(self, source_id: str, profile_name: str):
        """
        Applies a given profile to a given source.

        Parameters
        ----------
        source_id: str
            Name of the source to which to apply the given profile
        profile_name: str
            Name of the profile to apply to the given source.

        Returns
        -------
        None

        Raises
        -------
        ProfileNotFound
        SourceNotFound
        """
        if self.profile_manager.is_profile(profile_name):
            self.source_manager.apply_profile_to_source(source_id, profile_name)
        else:
            raise ProfileNotFound(profile_name)

    def apply_profile_to_sources(self, source_ids: List[str], profile_name: str):
        """
        Applies a given profile to a given list of sources.

        Parameters
        ----------
        source_ids: List[str]
            List of names of the sources to which to apply the given profile.
        profile_name: str
            The profile to be applied.

        Returns
        -------
        None

        Raises
        -------
        ProfileNotFound
        SourceNotFound
        """
        for source in source_ids:
            self.apply_profile_to_source(source, profile_name)

    def default_profile_name(self) -> str:
        """
        Get the name of current default profile.

        Returns
        -------
        str:
            The name of current default profile.
        """
        return self.profile_manager.default_profile_name

    def suite_has_new_version(self, suite_name: str) -> bool:
        """
        Check if a suite has a new version that can be updated

        Parameters
        ----------
        suite_name

        Returns
        -------
        bool
        """
        return self.plugin_suite_manager.has_new_version(suite_name)

    def modified_files_in_suite(self, suite_name: str) -> List[str]:
        """
        Check if a suite has been modified locally

        Parameters
        ----------
        suite_name

        Returns
        -------
        bool
        """
        return self.plugin_suite_manager.modified_files_in_suite(suite_name)

    def suite_copy_modified_suite(self, suite_name: str):
        return self.plugin_suite_manager.copy_modified_suite(suite_name)

    def register_suite(self, plugin_path: str) -> List[str]:
        """
        Registers a gailbot plugin suite.

        Parameters
        ----------
        plugin_path: str
            Path of the plugin suite to register.

        Returns
        -------
        List[str]:
            A list of plugin names if the plugin is registered.

        Raises
        -------
        FailPluginRegister
        """
        registered = self.plugin_suite_manager.register_suite(plugin_path)
        if not registered:
            raise FailPluginSuiteRegister(
                plugin_path,
                self.plugin_suite_manager.report_registration_err(plugin_path),
            )
        else:
            return registered

    def is_suite(self, name: str) -> bool:
        """
        Determines if a given name is an existing plugin suite.

        Parameters
        ----------
        name: str
            Name of the plugin suite to search for.

        Returns
        -------
        Bool:
            True if given plugin suite exists, false if not
        """
        return self.plugin_suite_manager.is_suite(name)

    def remove_suite(self, name: str):
        """
        Removes the given plugin suite.

        Parameters
        ----------
        name: str
            Name of the plugin suite to delete.

        Returns
        -------
        None

        Raises
        -------
        PluginSuiteNotFound
        RemoveInUseError
        """
        if self.profile_manager.is_suite_used_by_profile(name):
            raise RemoveInUseError(name)
        self.plugin_suite_manager.delete_suite(name)

    def get_selectable_plugins(self, suite_name: str) -> List[str]:
        """
        Given a suite_name, returns a list of selectable plugins from that
        suite
        Parameters
        ----------
        suite_name

        Returns
        -------

        Raises
        ------
        PluginSuiteNotFound
        """
        return self.plugin_suite_manager.get_selectable_plugins(suite_name)

    def available_suites(self) -> List[str]:
        """
        Get the names of available plugin suites.

        Returns
        -------
        List[str]:
            A list of available plugin suites name.
        """
        
        if self.plugin_suite_manager: 
            return self.plugin_suite_manager.get_all_suites_name() 
        else: 
            return []

    def suite_metadata(self, suite_name: str) -> Suite:
        """
        Get the metadata of a plugin suite identified by suite name.

        Parameters
        ----------
        suite_name: str
            The name of the suite.

        Returns
        -------
        MetaData: a MetaData object that stores the suite's metadata

        Raises
        ------
        PluginSuiteNotFound
        """
        return self.plugin_suite_manager.get_suite_metadata(suite_name)

    def suite_dependency_graph(self, suite_name: str) -> Dict[str, List[str]]:
        """
        Get the dependency map of the plugin suite identified by suite_name

        Parameters
        ----------
        suite_name: str
            The name of the suite.

        Returns
        -------
        Dict[str, List[str]]:
            The dependency graph of the suite.

        Raises
        -------
        PluginSuiteNotFound
        """
        return self.plugin_suite_manager.get_suite_dependency_graph(suite_name)

    def suite_documentation_path(self, suite_name: str) -> str:
        """
        Get the path to the documentation map of the plugin suite identified
        by suite_name.

        Parameters
        ----------
        suite_name: str
            The name of the suite.

        Returns
        -------
        str:
            The path to the documentation file.

        Raises
        -------
        PluginSuiteNotFound
        """
        return self.plugin_suite_manager.get_suite_documentation_path(suite_name)

    def is_active_suite(self, suite_name: str) -> bool:
        """
        Given a suite name, check if this suite is used in any of the profiles.

        Parameters
        ----------
        suite_name: str
            The name of the plugin suite.

        Returns
        -------
        bool:
            Return true if the suite is used in any of the profiles,
            false otherwise (including if the provided suite name doesn't exist).
        """
        return self.profile_manager.is_suite_used_by_profile(suite_name)

    def is_official_suite(self, suite_name: str) -> bool:
        """
        Check if the suite identified by the given suite name is official.

        Parameters
        ----------
        suite_name: str
            The name of the suite.

        Returns
        -------
        bool:
            True if the suite is official, false otherwise.

        Raises
        ------
        PluginSuiteNotFound
        """
        return self.plugin_suite_manager.is_official_suite(suite_name)

    def suite_code_path(self, suite_name: str) -> str:
        """
        Given the name of a plugin suite, return the path to the source
        code of the suite.
        If the suite name doesn't correspond to a valid directory,
        the key value pair is erased from the list of suite names
        and corresponding PluginSuite objects. TODO: Does this happen?

        Parameters
        ----------
        suite_name: str
            The name of the suite.

        Returns
        -------
        string:
            String representing path to the suite source code.

        Raises
        -----
        PluginSuiteNotFound
        """
        return self.plugin_suite_manager.get_suite_path(suite_name)

    def report_suite_registration_error(self, suite_source) -> str:
        """
        Given the original suite_source that is used during suite registration,
        return the error string that causes the string registration failure

        Parameter
        ---------
            suite_source (str): a string to the initial source of the suite

        Returns
        --------
            str: a string that describes the error that causes the suite registration to fail

        Raises
        --------
        FailReportError
        """
        return self.plugin_suite_manager.report_registration_err(suite_source)

    def available_engines(self) -> List[str]:
        """
        Get the names of all available engines.

        Returns
        ----------
        List[str]:
            A list of all available engine names.
        """
        return self.engine_manager.available_engines

    def add_engine(self, name: str, setting: Dict[str, str]) -> bool:
        """
        Add a new engine.

        Parameters
        ----------
        name: str
            The name of the new engine
        setting: Dict[str, str]
            Settings for the new engine

        Returns
        ----------
        bool:
            True if the engine is successfully created, false otherwise
        """
        return self.engine_manager.add_engine(name, setting)

    def remove_engine(self, name: str):
        """
        Remove the engine identified by name

        Parameters
        ----------
        name: str
            the name of the engine to be removed

        Returns
        ----------
        bool:
            True if the engine is successfully removed

        Raises
        ----------
        EngineNotFound
        RemoveInUseError
        """
        if self.profile_manager.is_engine_used_by_profile(name):
            raise RemoveInUseError(name)
        self.engine_manager.remove_engine(name)

    def update_engine(self, name: str, new_setting: Dict[str, str]) -> bool:
        """
        Update an engine's setting

        Parameters
        ----------
        name: str
            the name of the engine
        new_setting: Dict[str, str]
            the new setting of the engine

        Returns
        ----------
        bool:
            True if the engine's setting is successfully updated,
            false otherwise.

        Raises
        -----------
        EngineNotFound
        """
        return self.engine_manager.update_engine(name, new_setting)

    def get_engine_setting(self, name: str) -> Dict[str, str]:
        """
        Access the setting data of the given engine.

        Parameters
        ----------
        name: str
            the name of the engine

        Returns
        ----------
        Dict[str, str]:
            The content of the engine's setting stored in a dictionary

        Raises
        ------
        EngineNotFound
        """
        return self.engine_manager.get_engine_setting(name)

    def is_engine(self, name: str) -> bool:
        """
        Check if the given name is an existing engine

        Parameters
        ----------
        name: str
            the engine name to search for.

        Returns
        ----------
        bool:
            True if provided engine name is a valid engine, false otherwise
        """
        return self.engine_manager.is_engine(name)

    def default_engine_name(self) -> str:
        """
        Get the default engine's name

        Returns
        -------
        str:
            The default engine's name
        """
        return self.engine_manager.default_name

    def reload_suite(self, suite_name):
        """
        re-load the suite from the original source

        Parameters
        ----------
        suite_name: str
            the name of the suite

        Returns
        ----------

        Raises
        ------
        FailPluginSuiteReload
        """
        self.plugin_suite_manager.reload_suite(suite_name)

    @staticmethod
    def compatible_media_formats() -> List[str]:
        """
        Returns
        -------
        A list of media file formats supported by gailbot
        """
        return ConvertFun.compatible_formats()

    @staticmethod
    def compatible_audio_formats():
        """
        Returns
        -------
        A list of audio file formats supported by gailbot
        """
        return ConvertFun.compatible_audio_formats()

    @staticmethod
    def compatible_video_formats():
        """
        Returns
        -------
        A list of video file formats supported by gailbot
        """
        return ConvertFun.compatible_video_formats()

    def add_progress_watcher(self, source_id: str, watcher: Watcher):
        """
        Add a function displayer to track for the progress of source.

        Parameters
        ----------
        source_id: str
            The name of the source.
        watcher: Watcher
            Displayer is a function that takes in a string as argument, and the
            string encodes the progress of the source.

        Returns
        -------
        None

        Raises
        ------
        SourceNotFound
        """
        self.source_manager.add_progress_watcher(source_id, watcher)

    def get_source_obj(self, source_id: str):
        """
        return the source object identified by source_id
        Parameters
        ----------
        source_id

        Returns
        -------

        Raises
        ------
        SourceNotFound

        """
        src_obj = self.source_manager.get_source(source_id)
        if self.profile_manager.is_profile(src_obj.profile_name):
            applied_profile_name = src_obj.profile_name
        else:
            logger.warning(
                f"Applied profile {src_obj.profile_name} is deleted, use default profile instead"
            )
            applied_profile_name = self.profile_manager.default_profile_name
            self.apply_profile_to_source(
                source_id=source_id, profile_name=applied_profile_name
            )
        profile = self.profile_manager.get_profile_object(applied_profile_name)
        src_obj.initialize_profile(profile)
        return src_obj
    
    def get_suite_url_from_id(self, id):
        return self.plugin_suite_manager.get_suite_url_from_id(id)



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-07-17 19:21:51
# @Last Modified by:   Jason Y. Wu
# @Last Modified time: 2023-08-01 05:37:54
from typing import TypedDict, Dict, Any
from .resultInterface import ResultInterface
from gailbot.configs import get_format_md_path
from gailbot.shared.utils.general import copy
from gailbot.shared.utils.logger import makelogger

logger = makelogger("format_result")


class FormatResultDict(TypedDict):
    process_stats: Dict[str, str]


class FormatResult(ResultInterface):
    """
    Defines a class for the format result
    """

    def __init__(self, workspace: str, data: Dict[str, Any] = None) -> None:
        super().__init__(workspace, data)
        self.data = data

    def save_data(self, data: Dict[str, Any]):
        """
        Saves the inputted data

        Args:
            data: Dict[str, ProcessingStats]: data to save, in the form
            of a dictionary mapping strings to processing stats

        Returns:
            Bool: True if successfully saved, false if not
        """
        try:
            self.data = data
            return True
        except Exception as e:
            return False

    def get_data(self):
        """
        Accesses and object's data

        Returns:
            Data in the form Dict[str, ProcessingStats]
        """
        return self.data

    def output(self, path: str) -> bool:
        """TODO: currently no data will be written as format result"""
        try:
            FORMAT_MD = get_format_md_path()
            copy(FORMAT_MD, path)
            return True
        except Exception as e:
            logger.error(e, exc_info=e)
            return False



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-31 16:53:33
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-02 19:12:31

from typing import Dict, List, Tuple

from pyannote.core import Segment
from gailbot.shared.utils.logger import makelogger

_DEFAULT_SPEAKER = 0
logger = makelogger("parsers")


def parse_into_word_dicts(transcription: Dict) -> List[Dict]:
    """
    Parse the results of the transcription into a list of dictionaries
    containing the speaker, start time, end time, and text.

    Format of the transcription is detailed here: https://github.com/linto-ai/whisper-timestamped
    """
    parsed = list()
    segments = transcription["segments"]
    try:
        for segment in segments:
            if not "words" in segment:
                continue
            word_list = segment["words"]
            for item in word_list:
                parsed.append(
                    {
                        "start": item["start"],
                        "end": item["end"],
                        "text": item["text"],
                        # NOTE: Whisper does not generate speaker, but I can
                        # potentially add that in too.
                        "speaker": str(_DEFAULT_SPEAKER),
                    }
                )
                assert parsed
    except Exception as e:
        logger.error(f"getting the error from parsing word into dict", exc_info=e)
    return parsed


def parse_into_timestamped_text(asr_res: Dict) -> List[Tuple]:
    """
    Parse results from whisper timestamped in terms of Segment
    """
    timestamp_texts = []
    for segment in asr_res["segments"]:
        if not "words" in segment:
            continue
        word_list = segment["words"]
        for item in word_list:
            start = item["start"]
            end = item["end"]
            text = item["text"]
            timestamp_texts.append((Segment(start, end), text))
    return timestamp_texts


def parse_into_full_text(asr_res: Dict) -> str:
    """
    Parse the transcription output into a string.
    """
    return asr_res["text"]


def add_speaker_info_to_text(asr_res: Dict, dir_res: Dict) -> List[Dict]:
    """
    Add speaker information to transcription results using speaker
    diarization results. Returns dictionaries
    """
    spk_text = []
    timestamp_texts = parse_into_timestamped_text(asr_res)
    for seg, text in timestamp_texts:
        spk = dir_res.crop(seg).argmax()
        spk_text.append(
            {"start": seg.start, "end": seg.end, "speaker": spk, "text": text}
        )
    return spk_text



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-08 16:28:03
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-07 09:26:16
from enum import Enum
from typing import List, Dict
import psutil
import os
import glob
import shutil
import itertools
from pathlib import Path
import subprocess
from gailbot.shared.utils.logger import makelogger

import json
import yaml
import toml
import csv

logger = makelogger("general")


class CMD_STATUS(Enum):
    RUNNING = 0
    FINISHED = 1
    STOPPED = 2
    ERROR = 3
    NOTFOUND = 4


def is_directory(dir_path: str) -> bool:
    """
    Determine if the given path is a directory.

    Args:
        dir_path (str): directory path to check

    Returns:
        bool: true if the directory path is valid, false if not
    """
    try:
        return Path(dir_path).is_dir()
    except Exception as e:
        logger.error(dir_path, exc_info=e)
        return False


def is_file(file_path: str) -> bool:
    """
    Determine if the given path is a file.

    Args:
        file_path (str): file path to check

    Returns:
        bool: true if the file path is valid, false if not
    """
    try:
        return Path(file_path).is_file()
    except Exception as e:
        logger.error(file_path, exc_info=e)
        return False


def num_items_in_dir(
    path: str, extensions: List[str], recursive: bool = False, only_dirs: bool = False
) -> int:
    """
    Determine the number of files in the directory.

    Args:
        path (str): Path to the directory.
        extensions (List[str]): Specific file extensions to look for.
            Ex: ["pdf"]. '*' is a wildcard and considers all extensions.
            Does not consider sub-directories. Default is ["*"].
        recursive (bool): True to check subdirectories, False otherwise.
            Default is False.
        only_dirs (bool): Only applies to directories.

    Returns:
        int: number of files in the directory
    """
    try:
        return len(paths_in_dir(path, extensions, recursive, only_dirs))
    except Exception as e:
        logger.error(e)
        return False


def is_path(source: str) -> bool:
    """
    Checks whether a given string is a valid path.

    Args:
        source (str): the path to check if it is valid or not

    Returns:
        bool: true if the path is a file or directory
    """
    return is_file(source) or is_directory(source)


def paths_in_dir(
    path: str,
    filter: List[str] | None = None,
    recursive: bool = False,
) -> List[str]:
    """
    Determine the paths, relative to dir_path, of all files in the directory.
    Args:
        path (str): Path to the directory.
        filter (List[str]): Specific file extensions to look for, if filter is None, it looks for all file
        recursive (bool): True to check subdirectories, False otherwise.
            Default is False.
    Returns:
        List[str]: paths of all files in the directory
    """
    if not is_directory(path):
        logger.error("not a valid directory")
        return []
    if recursive:
        paths = []
        for root, dirs, files in os.walk(path):
            for file_name in files:
                if not filter or get_extension(file_name) in filter:
                    sub_path = os.path.join(root, file_name)
                    paths.append(sub_path)
    else:
        paths = []
        for file in os.listdir(path):
            if file[0] != ".":
                sub_path = os.path.join(path, file)
                if not filter or get_extension(sub_path) in filter:
                    paths.append(sub_path)
    return paths


def get_subfiles(path):
    subfiles = []

    # Walk through the directory recursively
    for root, dirs, files in os.walk(path):
        for file in files:
            # Get the full path to the file
            file_path = os.path.join(root, file)
            subfiles.append(file_path)

    return subfiles


def subdirs_in_dir(dir_path: str, recursive: bool = False) -> List[str]:
    """
    Get paths of subdirectories in the directory.

    Args:
        dir_path (str): Directory path to search for subdirectories.
        recursive (bool): True to check subdirectories, False otherwise.
            Default is False.

    Returns:
        List[str]: List of paths to subdirectories in the directory
    """
    try:
        return paths_in_dir(path=dir_path, filter=None, recursive=recursive)
    except Exception as e:
        logger.error(e)
        return []


def num_subdirs(dir_path: str, recursive: bool = False) -> int:
    """
    Get the number of subdirectories in the directory.

    Args:
        dir_path (str): Directory path to count subdirectories in.
        recursive (bool): True to check subdirectories, False otherwise.
            Default is False.

    Returns:
        int: Number of subdirectories in the directory
    """
    try:
        return len(subdirs_in_dir(dir_path, recursive))
    except Exception as e:
        logger.error(e)
        return False


def get_name(path: str) -> str:
    """
    Given the path, returns the name of the file or directory without extension.

    Args:
        path (str): Path to a file or directory.

    Returns:
        str: Name of the file or directory without extension.
    """
    try:
        dir_path, file_name = os.path.split(path)
        if file_name:
            return os.path.splitext(os.path.basename(path))[0]
        else:
            return os.path.basename(dir_path)
    except Exception as e:
        logger.error(e)
        return False


def get_extension(path: str) -> str:
    """
    Given the path to the file, return the extension of the file.

    Args:
        path (str): Path to a file.

    Returns:
        str: Extension of the file.
    """
    try:
        return os.path.splitext(os.path.basename(path))[1][1:]
    except Exception as e:
        logger.error(e)
        return False


def get_parent_path(path: str) -> str:
    """
    Given the path to the file, returns the path to the file's parent directory.

    Args:
        path (str): Path to a file.

    Returns:
        str: Path to the file's parent directory.
    """
    try:
        return str(Path(path).parent.absolute())
    except Exception as e:
        logger.error(e)
        return False


def get_size(path: str) -> bytes:
    """
    Given the path to the file, return the file size in bytes.

    Args:
        path (str): Path to a file or directory.

    Returns:
        bytes: Size of the file in bytes.
    """
    try:
        if is_file(path):
            return os.path.getsize(path)
        else:
            return sum([os.path.getsize(p) for p in paths_in_dir(path, recursive=True)])
    except Exception as e:
        logger.error(e)
        return False


def make_dir(path: str, overwrite: bool = False):
    """
    Given the path, create a directory.

    Args:
        path (str): Path to the directory to be created.
        overwrite (bool): True to overwrite the directory if it exists, False otherwise.

    Returns:
        None
    """
    try:
        if is_directory(path) and overwrite:
            delete(path)
        os.makedirs(path, exist_ok=True)
    except Exception as e:
        logger.error(e)
        return False


def move(src_path: str, tgt_path: str) -> str:
    """
    Move the file from the source path to the target path.

    Args:
        src_path (str): Source path of the file to be moved.
        tgt_path (str): Target path where the file will be moved to.

    Returns:
        str: The new path of the moved file.
    """
    try:
        return shutil.move(src_path, tgt_path)
    except Exception as e:
        logger.error(e)
        return False


def copy(src_path, tgt_path: str) -> str:
    """
    Copy the file from the source path to the target path.

    Args:
        src_path (str): Source path of the file to be copied.
        tgt_path (str): Target path where the file will be copied to.

    Returns:
        str: The new path of the copied file.
    """
    try:
        if is_file(src_path):
            return shutil.copy2(src_path, tgt_path)
        elif is_directory(src_path):
            return shutil.copytree(src_path, tgt_path, dirs_exist_ok=True)
        else:
            logger.error("not a valid file path")
    except Exception as e:
        logger.error(e)
        return False


def rename(src_path, new_name: str) -> str:
    """
    Rename the file in the source path to the new name.

    Args:
        src_path (str): Source path of the file to be renamed.
        new_name (str): New name for the file.

    Returns:
        str: The new path of the renamed file.
    """
    try:
        return str(Path(src_path).with_name(new_name).resolve())
    except Exception as e:
        logger.error(e)
        raise FileExistsError


def delete(path: str) -> None:
    """
    Given a path, delete the file or directory.

    Args:
        path (str): Path to the file or directory to be deleted.

    Returns:
        None
    """
    try:
        if is_file(path):
            Path(path).unlink(missing_ok=True)
        elif is_directory(path):
            shutil.rmtree(path)
        else:
            logger.error("not a valid path")
            return False
    except Exception as e:
        logger.error(e)
        return False


def read_json(path: str) -> List[dict]:
    """
    Given a path, read the JSON data stored in the file and return it as a dictionary.

    Args:
        path (str): Path to the JSON file.

    Returns:
        Dict: Dictionary containing the JSON data.
    """
    if is_file(path):
        with open(path, "r") as f:
            return json.load(f)
    else:
        raise FileExistsError


def write_json(path: str, data: List[Dict], overwrite: bool = True) -> None:
    """
    Given the path to a file and a list of dictionaries, output the data to the file in JSON format.

    Args:
        path (str): Path to the JSON file.
        data (List[Dict]): List of dictionaries to be stored in the JSON file.
        overwrite (bool): True to overwrite the file if it exists, False to update it.

    Returns:
        None
    """
    if not overwrite:
        d = read_json(path)
        d.update(data)
    else:
        d = data
    try:
        with open(path, "w+") as f:
            json.dump(d, f)
    except Exception as e:
        raise Exception(e)


def read_txt(path: str) -> List:
    """
    Given the path to a text file, return the content of the file as a list of strings.

    Args:
        path (str): Path to the text file.

    Returns:
        List: List of strings containing the file's content.
    """
    if is_file(path):
        with open(path, "r") as f:
            text = f.readlines()
        text = [s.strip() for s in text]
        return text
    else:
        raise FileExistsError


def write_txt(path: str, data: List, overwrite: bool = True) -> bool:
    """
    Given the path to a text file and a list of data, output the list data to the file.

    Args:
        path (str): Path to the text file.
        data (List): List of data to be written to the file.
        overwrite (bool): True to overwrite the file if it exists, False to append to it.

    Returns:
        bool: True if the operation is successful, False otherwise.
    """
    mode = "w+" if overwrite else "a"
    data = [s + "\n" for s in data]
    try:
        with open(path, mode) as f:
            f.writelines(data)
    except Exception as e:
        raise FileExistsError


def read_yaml(path: str) -> Dict:
    """
    Given a path to a YAML file, return a dictionary representation of the data stored in the YAML file.

    Args:
        path (str): Path to the YAML file.

    Returns:
        Dict: Dictionary containing the data from the YAML file.
    """

    if is_file(path):
        try:
            with open(path, "r") as f:
                data = yaml.safe_load(f)
                if not data:
                    data = yaml.unsafe_load(path)
                if not isinstance(data, dict):
                    logger.error(
                        f"the data is not a valid dictionary: {data}, the file path is {path}"
                    )
                    raise Exception
                return data
        except Exception as e:
            logger.error(e, exc_info=e)
    else:
        logger.error("not a valid YAML file")
        raise FileExistsError


def write_yaml(path: str, data: Dict, overwrite: bool = True) -> bool:
    """
    Given a path to a YAML file and data stored in a dictionary, output the data in YAML format to the file.

    Args:
        path (str): Path to the YAML file.
        data (Dict): Dictionary containing the data to be written to the YAML file.
        overwrite (bool): True to overwrite the file if it exists, False to update it.

    Returns:
        bool: True if the operation is successful, False otherwise.
    """
    data = dict(data)
    if not overwrite:
        previous_data = read_yaml(path)
        previous_data.update(data)
        previous_data.update(data)
    else:
        previous_data = data
    try:
        with open(path, "w+") as f:
            yaml.dump(previous_data, f)
    except Exception as e:
        raise FileExistsError


def read_toml(path: str) -> Dict:
    """
    Given the path to a TOML file, return a dictionary representation of the data stored in the TOML file.

    Args:
        path (str): Path to the TOML file.

    Returns:
        Dict: Dictionary containing the data from the TOML file.
    """
    if is_file(path):
        return toml.load(path)
    else:
        logger.error("not a valid TOML file")
        raise FileExistsError


def write_toml(path: str, data: Dict) -> bool:
    """
    Given the path to a TOML file and data stored in a dictionary, output the data in TOML format to the file.

    Args:
        path (str): Path to the TOML file.
        data (Dict): Dictionary containing the data to be written to the TOML file.

    Returns:
        bool: True if the operation is successful, False otherwise.
    """
    try:
        with open(path, "w+") as f:
            toml.dump(data, f)
    except Exception as e:
        raise FileExistsError


def write_csv(path: str, data: List[Dict[str, str]]):
    """
    Write data to a CSV file.

    Args:
        path (str): Path to the CSV file.
        data (List[Dict[str, str]]): List of dictionaries to be written to the CSV file.
    """
    if not data or len(data) == 0:
        with open(path, mode="w+", newline="") as file:
            pass
        return
    fields = list(data[0].keys())
    # Write the data to the CSV file
    with open(path, mode="w+", newline="") as file:
        writer = csv.DictWriter(file, fieldnames=fields)
        writer.writeheader()
        for row in data:
            writer.writerow(row)


def read_csv(path: str) -> List[Dict[str, str]]:
    """
    Read data from a CSV file and return it as a list of dictionaries.

    Args:
        path (str): Path to the CSV file.

    Returns:
        List[Dict[str, str]]: List of dictionaries containing the data from the CSV file.
    """
    data = []
    with open(path, mode="r") as file:
        reader = csv.DictReader(file)
        for row in reader:
            data.append(row)
    return data


def run_cmd(cmd: List[str]) -> int:
    """
    Run a shell command and obtain a process identifier.

    Args:
        cmd (List[str]): A list of strings that stores the command.

    Returns:
        int: the return code of the process
    """
    cmd = " ".join(cmd)
    result = subprocess.run(cmd, shell=True, capture_output=True)
    return result.returncode


def get_cmd_status(identifier: int) -> CMD_STATUS:
    """
    Obtain the status of the shell command associated with the given identifier.

    Args:
        identifier (int): The process ID of a running process.

    Returns:
        CMD_STATUS: A string representing the process status.
    """
    try:
        process = psutil.Process(identifier)
        status = process.status()
        match status:
            case "zombie":
                return CMD_STATUS.FINISHED
            case "running":
                return CMD_STATUS.RUNNING
            case "stopped":
                return CMD_STATUS.RUNNING
            case other:
                return CMD_STATUS.ERROR
    except psutil.NoSuchProcess:
        return CMD_STATUS.NOTFOUND


def copy_dir_files(src_folder, dest_folder):
    """
    Copy files from the source folder to the destination folder recursively.

    Args:
        src_folder (str): The path to the source folder.
        dest_folder (str): The path to the destination folder.
    """
    if not os.path.exists(dest_folder):
        os.makedirs(dest_folder)

    # Walk through the source directory tree
    for root, dirs, files in os.walk(src_folder):
        # Create the corresponding subdirectories in the destination folder
        dest_root = root.replace(src_folder, dest_folder)
        for dir_name in dirs:
            dest_dir = os.path.join(dest_root, dir_name)
            if not os.path.exists(dest_dir):
                os.makedirs(dest_dir)

        # Copy the files to the destination folder
        for filename in files:
            src_path = os.path.join(root, filename)
            dest_path = os.path.join(dest_root, filename)
            shutil.copy2(src_path, dest_path)



import requests
from threading import Lock

class APIConsumer:
    _instance = None
    _lock = Lock()

    def __new__(cls, user_id, token):
        if cls._instance is None:
            print(f"new instance userid: {user_id}, token: {token}")
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super(APIConsumer, cls).__new__(cls)
                    cls._instance.user_id = user_id
                    cls._instance.base_url = "https://www.gailbot.ai"
                    cls._instance.headers = {'Authorization': f'Bearer {token}'}
        return cls._instance
        
    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            raise ValueError("Must initialize first")
        return cls._instance

    @classmethod
    def reset_instance(cls):
        with cls._lock:
            cls._instance = None

    def fetch_plugin_info(self, plugin_id):
        user_id = self.user_id
        if user_id is None:
            raise ValueError("User ID must be set via setUserId or passed explicitly.")

        endpoint = f"{self.base_url}/api/plugins/{plugin_id}"
        try:
            response = requests.get(endpoint, headers=self.headers)
            response.raise_for_status()
            data = response.json()
            matching = next(
                (v for v in data.get("versions", [])
                if v.get("id") == data.get("id")),
                None
            )
            return matching
        except requests.RequestException as e:
            print(f"Error fetching plugin info: {e}")
            return None

    def fetch_suite_info(self, suite_id):
        user_id = self.user_id
        if user_id is None:
            raise ValueError("User ID must be set via setUserId or passed explicitly.")

        endpoint = f"{self.base_url}/api/suites/{suite_id}"
        try:
            response = requests.get(endpoint, headers=self.headers)
            response.raise_for_status()
            data = response.json()
            matching = next(
                (v for v in data.get("versions", [])
                if v.get("id") == data.get("id")),
                None
            )
            return matching
        except requests.RequestException as e:
            print(f"Error fetching suite info: {e}")
            return None
        
    def fetch_secrets(self):
        endpoint = f"{self.base_url}/api/fetch-secrets"
        try:
            response = requests.get(endpoint, headers=self.headers)
            response.raise_for_status()
            response_json = response.json()
            print(f"Response JSON: {response_json}")
            return response_json
        except requests.RequestException as e:
            print(f"Error fetching suite info: {e}")
            return None



from pydantic import BaseModel
from typing import List
from gailbot.setting_interface.formItem import FormItem, FormType

class DeepgramSetting(BaseModel):
    engine: str = "deepgram"
    deepgram_api_key: str
    language: str = "en-US"
    punctuate: bool = True
    diarize: bool = False
    profanity_filter: bool = False
    model: str = "general"
    tier: str = "standard"

    @staticmethod
    def get_setting_config() -> List[FormItem]:
        return [
            FormItem(type=FormType.Selection, name="engine", selection_items=["deepgram"], default_value="deepgram"),
            FormItem(type=FormType.Text, name="deepgram_api_key"),
            FormItem(type=FormType.Selection, name="language", selection_items=["en-US", "es-ES", "fr-FR", "de-DE"], default_value="en-US"),
            FormItem(type=FormType.OnOff, name="punctuate", default_value=True),
            FormItem(type=FormType.OnOff, name="diarize", default_value=False),
            FormItem(type=FormType.OnOff, name="profanity_filter", default_value=False),
            FormItem(type=FormType.Selection, name="model", selection_items=[
                "nova-3",
                "nova-3-general",
                "nova-3-medical",
                "nova-2",
                "nova-2-general",
                "nova-2-meeting",
                "nova-2-phonecall",
                "nova-2-voicemail",
                "nova-2-finance",
                "nova-2-conversationalai",
                "nova-2-video",
                "nova-2-medical",
                "nova-2-drivethru",
                "nova-2-automotive",
                "enhanced-general",
                "base-general"
            ], default_value="base-general"),
        ]


# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-02-06 10:19:08
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-02-06 15:35:20
import os.path
from gailbot.api import GailBot
from gailbot.sourceManager.sourceObject import Watcher

OUTPUT = "/Users/yike/Desktop/gbtest/output"
TEST_ROOT = "/Users/yike/Desktop/gbtest/input"


class TestWatcher(Watcher):
    def watch(self, progress: str):
        print(progress)


def test_parallel_transcription():
    gb = GailBot()
    source_ids = [
        gb.add_source(
            source_path=os.path.join(TEST_ROOT, "hello2.wav"), output_dir=OUTPUT
        )
        for _ in range(4)
    ]
    source_ids.extend(
        [
            gb.add_source(
                source_path=os.path.join(TEST_ROOT, "short_test/shorttest.wav"),
                output_dir=OUTPUT,
            )
            for _ in range(4)
        ]
    )
    for id in source_ids:
        gb.add_progress_watcher(id, TestWatcher())
    result = gb.transcribe(source_ids)
    assert not result.failure



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-07-17 19:21:51
# @Last Modified by:   Jason Y. Wu
# @Last Modified time: 2023-08-01 05:38:05
from typing import Dict
from .processingStatus import ProcessingStats
from gailbot.shared.utils.logger import makelogger
from gailbot.shared.utils.general import write_json

logger = makelogger("result_interface")
""" .pickle , use efficient way to save the file """


class ResultInterface:
    """
    Defines a class containing the logic for transcription, format, and analysis results
    """

    def __init__(self, workspace: str, data=None) -> None:
        self.workspace = workspace
        self.data = data
        self.processingStats = None

    def set_processing_stats(self, stats: ProcessingStats):
        """
        Sets an object's processing stats

        Args:
            stats: ProcessingStats: ProcessingStats to set

        Returns:
            bool: True if successfully set, false if not
        """
        try:
            assert stats
            self.processingStats: ProcessingStats = stats
            return True
        except Exception as e:
            logger.error(e, exc_info=e)
            return False

    def output_processing_stats(self, output: str):
        """
        Outputs an object's processing stats to the output directory

        Args:
            output: str: Path of the output directory

        Returns:
            bool: True if successfully set, false if not
        """
        try:
            write_json(output, self.processingStats.__dict__)
            return True
        except Exception as e:
            logger.error(e, exc_info=e)
            return False

    def save_data(self, data) -> bool:
        raise NotImplementedError

    def output(self, path: str) -> bool:
        raise NotImplementedError

    def get_data(self) -> Dict:
        raise NotImplementedError
